/* Braidwood Nuclear Plant
   Heat Rejection Model
   developed by
   dudley.benton@gmail.com
*/
#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "Braidwood.h"
#include "..\\moistair\\moistair.h"
#include "..\\steam\\steam.h"

extern int __argc;
extern char**__argv;
extern char**_environ;

void Abort(int code,char*format,...)
  {
  char cbuf[1024],head[32];
  va_list arg_marker;
  va_start(arg_marker,format);
  vsprintf(cbuf,format,arg_marker);
  sprintf(head,"Fatal Error Code %li",code);
  MessageBox(NULL,cbuf,head,MB_ICONSTOP|MB_OK);
  ExitProcess(0);
  }

void WindowText(HWND hWnd,char*format,...)
  {
  char cbuf[129];
  int l;
  va_list arg_marker;
  va_start(arg_marker,format);
  l=vsprintf(cbuf,format,arg_marker);
  if(l>=sizeof(cbuf))
    Abort(__LINE__,"WindowText(): buffer overflow");
  SetWindowText(hWnd,cbuf);
  }

void DrawLine(HDC hDC,DWORD rgb,int width,int x1,int y1,int x2,int y2)
  {
  HPEN hPen;
  POINT P;
  hPen=CreatePen(PS_SOLID,width,rgb);
  hPen=SelectObject(hDC,hPen);
  MoveToEx(hDC,x1,y1,&P);
  LineTo(hDC,x2,y2);
  hPen=SelectObject(hDC,hPen);
  DeleteObject(hPen);
  }

void GetTextSize(HDC hDC,char*string,int len,SIZE*S)
  {
  static TEXTMETRIC Tm;
  S->cx=S->cy=0;
  if(len>0)
    {
    GetTextExtentPoint32(hDC,string,len,S);
    if(S->cx<6||S->cy<8)
      {
      GetTextMetrics(hDC,&Tm);
      if(Tm.tmAveCharWidth<6||Tm.tmHeight<10)
        {
        S->cx=((2*len+1)*6)/2;
        S->cy=10;
        }
      else
        {
        S->cx=((2*len+1)*Tm.tmAveCharWidth)/2;
        S->cy=(9*Tm.tmHeight)/8;
        }
      }
    }
  }

HFONT CreateStandardFont(char*name,int weight,double angle)
  {
  HFONT hFont;
  LOGFONT*lFont;
  lFont=(LOGFONT*)calloc(sizeof(LOGFONT),1);
  strcpy(lFont->lfFaceName,name);
  lFont->lfWeight=weight;
  lFont->lfEscapement=(int)(angle*10.+0.5);
  hFont=CreateFontIndirect(lFont);
  free(lFont);
  return(hFont);
  }

void PaintBitmap(HDC hDC,RECT*rc,BITMAPINFOHEADER*bHead,int stretch)
  {
  int colors,h,H,l,t,w,W;
  BYTE*bImage;
  l=rc->left;
  t=rc->top;
  w=rc->right-rc->left;
  h=rc->bottom-rc->top;
  W=bHead->biWidth;
  H=bHead->biHeight;
  if(w<8||h<8||W<8||H<8)
    return;
  if(bHead->biBitCount==24)
    colors=0;
  else
    {
    colors=bHead->biClrUsed;
    if(colors==0)
      colors=1<<bHead->biBitCount;
    }
  bImage=((BYTE*)bHead)+sizeof(BITMAPINFOHEADER)+colors*sizeof(DWORD);
  if(stretch<0)
    SetDIBitsToDevice(hDC,l,t,W,H,0,0,0,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS);
  else if(stretch>0)
    StretchDIBits(hDC,l,t,w,h,0,0,W,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS,SRCCOPY);
  else if(w*abs(H)<abs(W)*h)
    StretchDIBits(hDC,l,t+(h-w*abs(H)/abs(W))/2,w,w*abs(H)/abs(W),0,0,W,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS,SRCCOPY);
  else
    StretchDIBits(hDC,l+(w-h*abs(W)/abs(H))/2,t,h*abs(W)/abs(H),h,0,0,W,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS,SRCCOPY);
  }

int nint(double d)
  {
  if(d>0.)
    return((int)(d+0.5));
  if(d<0.)
    return((int)(d-0.5));
  return(0);
  }

/*******************************************************************************
  GLOBAL APPLICATION OBJECTS
*******************************************************************************/

HINSTANCE hInst;
HWND hMain;
HWND hPlot;
HWND bAbout;
HWND bExit;
HWND bHelp;
HWND pCondenser;
HWND pPond;
HWND pTurbine;
HWND pReactor;

HANDLE stdInput;
HANDLE stdOutput;

HFONT fBold;
HFONT fHorizontal;
HFONT fVertical;

BITMAPINFOHEADER*bCondenser;
BITMAPINFOHEADER*bPlant;
BITMAPINFOHEADER*bReactor;
BITMAPINFOHEADER*bPond;
BITMAPINFOHEADER*bTurbine;

char cbuf[1024];
int display_plot;

struct{
  POINT button;
  POINT client;
  POINT draw;
  POINT frame;
  POINT plant;
  POINT push;
  POINT screen;
  POINT window;
  }sizes;

DWORD color[16]={
  0x000000, /* black */
  0xA80000, /* blue */
  0x00A800, /* green */
  0xA8A800, /* cyan */
  0x0000A8, /* red */
  0xA800A8, /* magenta */
  0x00A8A8, /* brown */
  0xD0D0D0, /* white */
  0xA8A8A8, /* dark_gray */
  0xFC0000, /* light_blue */
  0x00FC00, /* light_green */
  0xFCFC00, /* light_cyan */
  0x0000FC, /* light_red */
  0xFC00FC, /* light_magenta */
  0x00FCFC, /* yellow */
  0xFCFCFC};/* bright_white */

struct{
  int    legend; /* legend location */
  int    curves; /* number of curves */
  char **label ; /* curve labels */
  int   *points; /* number of points along each curve */
  char  *Title ; /* plot title */
  char  *Xlabel; /* X axis label */
  double*Xp    ; /* X points */
  char  *Ylabel; /* Y axis label */
  double*Yp    ; /* Y points */
  }plot;

void PlotCurves(HDC hDC,RECT rWindow)
  {
  int c,i,p,x,x1,x2,y,y1,y2;
  double Ax,Ay,Bx,By,dX,dY,X,Xm,Xx,Y,Ym,Yx;
  HRGN hRgn;
  RECT rPlot;
  static SIZE sText,sTick,sXnum,sYnum;
  FillRect(hDC,&rWindow,GetStockObject(WHITE_BRUSH));
  for(i=0;i<2;i++)
    {
    FrameRect(hDC,&rWindow,GetStockObject(BLACK_BRUSH));
    rWindow.top++;
    rWindow.left++;
    rWindow.right--;
    rWindow.bottom--;
    }
  rWindow.left  +=2;
  rWindow.top   +=2;
  rWindow.right -=2;
  rWindow.bottom-=2;
  SelectClipRgn(hDC,NULL);
  SetBkMode(hDC,TRANSPARENT);
  SetTextColor(hDC,color[0]);
  SelectObject(hDC,fHorizontal);
  GetTextSize(hDC,plot.Title,strlen(plot.Title),&sText);
  TextOut(hDC,(rWindow.right-rWindow.left-sText.cx)/2,rWindow.top,plot.Title,strlen(plot.Title));
  SelectObject(hDC,fVertical);
  GetTextSize(hDC,plot.Ylabel,strlen(plot.Ylabel),&sText);
  TextOut(hDC,rWindow.left,(rWindow.bottom-rWindow.top+sText.cx)/2,plot.Ylabel,strlen(plot.Ylabel));
  SelectObject(hDC,fHorizontal);
  GetTextSize(hDC,plot.Xlabel,strlen(plot.Xlabel),&sText);
  TextOut(hDC,(rWindow.right-rWindow.left-sText.cx)/2,rWindow.bottom-sText.cy,plot.Xlabel,strlen(plot.Xlabel));
  if(plot.curves<1)
    Abort(__LINE__,"PlotCurves(): curves=%i <1",plot.curves);
  for(c=0;c<plot.curves;c++)
    if(plot.points[c]<2)
      Abort(__LINE__,"PlotCurves(): curve=%i, points=%i <2",c,plot.points[c]);
  Xm=Xx=plot.Xp[0];
  Ym=Yx=plot.Yp[0];
  for(c=i=0;c<plot.curves;c++)
    {
    for(p=0;p<plot.points[c];p++,i++)
      {
      Xm=min(Xm,plot.Xp[i]);
      Xx=max(Xx,plot.Xp[i]);
      Ym=min(Ym,plot.Yp[i]);
      Yx=max(Yx,plot.Yp[i]);
      }
    }
  if(Xm>=Xx)
    Abort(__LINE__,"PlotCurves(): X is degenerate");
  i=(int)(log10(Xx-Xm)-0.5);
  dX=pow(10,i);
  if(Xx-Xm>15*dX)
    dX*=5;
  else if(Xx-Xm>7*dX)
    dX*=2;
  X=Xm;
  Xm=dX;
  while(Xm<Xx)
    Xm+=dX;
  while(Xm>X)
    Xm-=dX;
  X=Xx;
  Xx=Xm;
  while(Xx<X)
    Xx+=dX;
  sXnum.cx=sXnum.cy=0;
  for(X=Xm;X<=Xx+dX/10;X+=dX)
    {
    sprintf(cbuf,"%lG",Xx);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    sXnum.cx=max(sXnum.cx,sText.cx);
    sXnum.cy=max(sXnum.cy,sText.cy);
    }
  if(Ym>=Yx)
    Abort(__LINE__,"PlotCurves(): Y is degenerate");
  i=(int)(log10(Yx-Ym)-0.5);
  dY=pow(10,i);
  if(Yx-Ym>15*dY)
    dY*=5;
  else if(Yx-Ym>7*dY)
    dY*=2;
  Y=Ym;
  Ym=dY;
  while(Ym<Yx)
    Ym+=dY;
  while(Ym>Y)
    Ym-=dY;
  Y=Yx;
  Yx=Ym;
  while(Yx<Y)
    Yx+=dY;
  sYnum.cx=sYnum.cy=0;
  for(Y=Ym;Y<=Yx+dY/10;Y+=dY)
    {
    sprintf(cbuf,"%lG",Yx);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    sYnum.cx=max(sYnum.cx,sText.cx);
    sYnum.cy=max(sYnum.cy,sText.cy);
    }
  GetTextSize(hDC,"0",1,&sTick);
  rPlot.left  =rWindow.left  +3*sTick.cx+sYnum.cx;
  rPlot.top   =rWindow.top   +sXnum.cy;
  rPlot.right =rWindow.right -sXnum.cx/2;
  rPlot.bottom=rWindow.bottom-sXnum.cy-sYnum.cy;
  Ax=(rPlot.right-rPlot.left)/(Xx-Xm);
  Bx=rPlot.left-Ax*Xm;
  Ay=(rPlot.top-rPlot.bottom)/(Yx-Ym);
  By=rPlot.bottom-Ay*Ym;
  for(X=Xm;X<Xx+dX/10;X+=dX)
    {
    x=nint(Ax*X+Bx);
    sprintf(cbuf,"%lG",X);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    TextOut(hDC,x-sText.cx/2,rPlot.bottom+sTick.cx/4,cbuf,strlen(cbuf));
    DrawLine(hDC,color[0],1,x,rPlot.bottom+sTick.cx/4,x,rPlot.top-1);
    }
  for(Y=Ym;Y<Yx+dY/10;Y+=dY)
    {
    y=nint(Ay*Y+By);
    sprintf(cbuf,"%lG",Y);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    TextOut(hDC,rPlot.left-sText.cx-3*sTick.cx/4,y-sText.cy/2,cbuf,strlen(cbuf));
    DrawLine(hDC,color[0],1,rPlot.left-sTick.cx/2,y,rPlot.right+3,y);
    }
  rPlot.left++;
  rPlot.top++;
  hRgn=CreateRectRgnIndirect(&rPlot);
  SelectClipRgn(hDC,hRgn);
  for(c=i=0;c<plot.curves;c++)
    {
    x2=nint(Ax*plot.Xp[i]+Bx);
    y2=nint(Ay*plot.Yp[i]+By);
    for(p=1,++i;p<plot.points[c];p++,i++)
      {
      x1=x2;
      y1=y2;
      x2=nint(Ax*plot.Xp[i]+Bx);
      y2=nint(Ay*plot.Yp[i]+By);
      DrawLine(hDC,color[1+c%14],1,x1,y1,x2,y2);
      }
    }
  SetBkMode(hDC,OPAQUE);
  if(plot.legend&1)
    x=(rPlot.left+3*rPlot.right)/4;
  else
    x=rPlot.left+sTick.cx;
  if(plot.legend&2)
    y=rPlot.bottom-(2*plot.curves+1)*sTick.cy/2;
  else
    y=rPlot.top+sTick.cy/2;
  for(c=0;c<plot.curves;c++,y+=sXnum.cy)
    {
    SetTextColor(hDC,color[1+c%14]);
    TextOut(hDC,x,y,plot.label[c],strlen(plot.label[c]));
    }
  DeleteObject(hRgn);
  }

int WINAPI DrawProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_CLOSE)
    return(FALSE);
  if(wMsg==WM_COMMAND)
    return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  if(wMsg==WM_CREATE)
    return(FALSE);
  if(wMsg==WM_DESTROY)
    return(FALSE);
  if(wMsg==WM_PAINT)
    {
    int i;
    HDC hDC;
    PAINTSTRUCT pS;
    RECT cR;
    if((hDC=BeginPaint(hWnd,&pS))==NULL)
      Abort(__LINE__,"DrawProc(): can't get device context\nWin Err code %i",GetLastError());
    GetClientRect(hWnd,&cR);
    if(display_plot)
      PlotCurves(hDC,cR);
    else
      {
      for(i=0;i<2;i++)
        {
        FrameRect(hDC,&cR,GetStockObject(BLACK_BRUSH));
        cR.top++;
        cR.left++;
        cR.right--;
        cR.bottom--;
        }
      PaintBitmap(hDC,&cR,bPlant,0);
      }
    EndPaint(hWnd,&pS);
    return(FALSE);
    }
  if(wMsg==WM_TIMER)
    return(FALSE);
  return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  }

void DeallocateCurves()
  {
  int c;
  if(plot.points)
    free(plot.points);
  if(plot.label)
    {
    for(c=0;c<plot.curves;c++)
      free(plot.label[c]);
    free(plot.label);
    }
  if(plot.Xp)
    free(plot.Xp);
  if(plot.Yp)
    free(plot.Yp);
  }

void PositionWindows()
  {
  int x,y;
  RECT rc;
  GetClientRect(hMain,&rc);
  sizes.draw.x=rc.right-sizes.push.x-sizes.frame.x;
  sizes.draw.y=rc.bottom;
  x=y=0;
  MoveWindow(hPlot,x,y,sizes.draw.x,sizes.draw.y,TRUE);
  x+=sizes.draw.x+sizes.frame.x;
  MoveWindow(pReactor,x,y,sizes.push.x,sizes.push.y,TRUE);
  y+=sizes.push.y+sizes.frame.y;
  MoveWindow(pTurbine,x,y,sizes.push.x,sizes.push.y,TRUE);
  y+=sizes.push.y+sizes.frame.y;
  MoveWindow(pPond,x,y,sizes.push.x,sizes.push.y,TRUE);
  y+=sizes.push.y+sizes.frame.y;
  MoveWindow(pCondenser,x,y,sizes.push.x,sizes.push.y,TRUE);
  y=rc.bottom-sizes.button.y;
  MoveWindow(bExit,x,y,sizes.button.x,sizes.button.y,TRUE);
  y-=sizes.button.y+sizes.frame.y;
  MoveWindow(bAbout,x,y,sizes.button.x,sizes.button.y,TRUE);
  y-=sizes.button.y+sizes.frame.y;
  MoveWindow(bHelp,x,y,sizes.button.x,sizes.button.y,TRUE);
  }

void CenterWindowOnScreen(HWND hWnd)
  {
  int high,left,top,wide;
  RECT wr;
  GetWindowRect(hWnd,&wr);
  wide=wr.right-wr.left;
  high=wr.bottom-wr.top;
  left=(GetSystemMetrics(SM_CXSCREEN)-wide)/2;
  top=(GetSystemMetrics(SM_CYSCREEN)-high)/2;
  MoveWindow(hWnd,left,top,wide,high,TRUE);
  }

LRESULT WINAPI AboutProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_CLOSE)
    return(FALSE);
  if(wMsg==WM_COMMAND)
    {
    if(HIWORD(wParam)==BN_CLICKED||HIWORD(wParam)==BN_DOUBLECLICKED)
      return(EndDialog(hWnd,LOWORD(wParam)));
    else
      return(FALSE);
    }
  if(wMsg==WM_CTLCOLORBTN)
    {
    SendMessage((HWND)lParam,WM_SETFONT,(WPARAM)GetStockObject(SYSTEM_FONT),0);
    SetBkMode((HDC)wParam,TRANSPARENT);
    return((LRESULT)GetStockObject(LTGRAY_BRUSH));
    }
  if(wMsg==WM_CTLCOLORDLG)
    return((LRESULT)GetStockObject(LTGRAY_BRUSH));
  if(wMsg==WM_CTLCOLORSTATIC)
    {
    SelectObject((HDC)wParam,GetStockObject(ANSI_VAR_FONT));
    SetBkMode((HDC)wParam,TRANSPARENT);
    SetTextColor((HDC)wParam,0x000000);
    return((LRESULT)GetStockObject(LTGRAY_BRUSH));
    }
  if(wMsg==WM_DESTROY)
    return(TRUE);
  if(wMsg==WM_INITDIALOG)
    {
    CenterWindowOnScreen(hWnd);
    BringWindowToTop(hWnd);
    return(TRUE);
    }
  return(FALSE);
  }

/*******************************************************************************
  COOLING POND OBJECTS
*******************************************************************************/

double Bpond=583.5; /* bottom of pond [ft above MSL] */
double Dpond=600.0; /* elevation of pond dike [ft above MSL] */
double Epond=595.0; /* pond elevation [ft above MSL] */
double Mpond=590.0; /* pond elevation required for CCW pumps */
double Vwind=  5.0; /* wind speed [mph] */
double solar=200.0; /* solar heating [BTU/hr/ft] */

double PondArea(double Elev)
  {
  if(Elev<=Bpond)
    return(0.);
  return(252.305*pow(Elev-Bpond,0.945033));
  }

double PondVolume(double Elev)
  {
  if(Elev<=Bpond)
    return(0.);
  return(252.305*pow(Elev-Bpond,1.945033)/1.945033);
  }

double InterpolateTable(double*X,double*Y,int n,double x)
  {
  int i,j;
  for(i=j=0;i<n-1;i++)
    {
    if((X[i]-x)*(X[i+1]-x)<=0.)
      {
      j=i+1;
      break;
      }
    }
  if(j==0)
    {
    if(fabs(x-X[0])<fabs(x-X[n-1]))
      j=1;
    else
      j=n-1;
    }
  return(Y[j-1]+(Y[j]-Y[j-1])*(x-X[j-1])/(X[j]-X[j-1]));
  }

double Pfactor(double dT1,double dT2) /* Langhaar's P factor */
  {
  return(57.5*log(dT1/dT2));
  }

double Qfactor(double T,double W) /* Langhaar's Q-factor from equilibrium */
  {                               /* temperature [F] and wind speed [mph] */
  double Yt,Yq,Yw;
  static double Xt=185,Xq=444,Xw=851;
  static double Ts[]={40, 50, 60, 70, 80, 90,100,110,120,130, 140};
  static double sT[]={85,163,250,361,463,563,672,777,886,992,1096};
  static double Ws[]={ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 12, 14, 16, 18 , 20};
  static double sW[]={91,173,243,313,372,435,485,537,587,632,679,766,834,903,976,1029};
  static double sQ[]={ 88,150,230,270,306,359,412,474,513,554,597,665,721,803,902,1041};
  static double Qs[]={3.0,2.5,2.0,1.8,1.6,1.4,1.2,1.0,0.9,0.8,0.7,0.6,0.5,0.4,0.3, 0.2};
  static i=0;
  if(i==0)
    for(;i<sizeof(Qs)/sizeof(double);i++)
      Qs[i]=log(Qs[i]);
  Yt=InterpolateTable(Ts,sT,sizeof(Ts)/sizeof(double),T);
  Yw=InterpolateTable(Ws,sW,sizeof(Ws)/sizeof(double),W);
  Yq=Yt+(Yw-Yt)*(Xq-Xt)/(Xw-Xt);
  return(exp(InterpolateTable(sQ,Qs,sizeof(Qs)/sizeof(double),Yq)));
  }

double Rheat(double T,double W) /* Langhaar's refrence heat transfer */
  {                             /* [BTU/hr/ft] from equilibrium */
  double Yt,Yw,Yh;              /* temp [F] and wind speed [mph] */
  static double Xt=136,Xw=537,Xh=831;
  static double Ts[]={ 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85,  90,  95, 100, 105, 110, 115, 120, 125, 130, 135, 140};
  static double sT[]={393,459,529,584,639,695,745,795,844,887,928,973,1018,1058,1099,1141,1181,1221,1262,1306,1346,1390,1433};
  static double Ws[]={ 20, 15, 10,  5,  0};
  static double sW[]={648,681,710,764,820};
  static double sH[]={  49,  85, 113, 152, 199, 255,283,319,361,412,439,471,507,545,590,642,663,693,718,745,776,814,853,950,1009,1049,1083,1131,1182,1214,1245,1280,1315,1362,1409};
  static double Hs[]={2000,1800,1600,1400,1200,1000,900,800,700,600,550,500,450,400,350,300,280,260,240,220,200,180,160,120, 100,  90,  80,  70,  60,  55,  50,  45,  40,  35,  30};
  static i=0;
  if(i==0)
    for(;i<sizeof(Hs)/sizeof(double);i++)
      Hs[i]=log(Hs[i]);
  Yt=InterpolateTable(Ts,sT,sizeof(Ts)/sizeof(double),T);
  Yw=InterpolateTable(Ws,sW,sizeof(Ws)/sizeof(double),W);
  Yh=Yt+(Yw-Yt)*(Xh-Xt)/(Xw-Xt);
  return(exp(InterpolateTable(sH,Hs,sizeof(Hs)/sizeof(double),Yh)));
  }

double Tequi(double H,double W) /* Langhaar's equilibrium temperature */
  {                             /* [F] from heat gain [BTU/hr/ft] */
  double Yt,Yw,Yh;              /* and wind speed [mph] */
  static double Xt=136,Xw=537,Xh=831;
  static double Ts[]={ 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85,  90,  95, 100, 105, 110, 115, 120, 125, 130, 135, 140};
  static double sT[]={393,459,529,584,639,695,745,795,844,887,928,973,1018,1058,1099,1141,1181,1221,1262,1306,1346,1390,1433};
  static double Ws[]={ 20, 15, 10,  5,  0};
  static double sW[]={648,681,710,764,820};
  static double Hs[]={2000,1800,1600,1400,1200,1000,900,800,700,600,550,500,450,400,350,300,280,260,240,220,200,180,160,120, 100,  90,  80,  70,  60,  55,  50,  45,  40,  35,  30};
  static double sH[]={  49,  85, 113, 152, 199, 255,283,319,361,412,439,471,507,545,590,642,663,693,718,745,776,814,853,950,1009,1049,1083,1131,1182,1214,1245,1280,1315,1362,1409};
  static i=0;
  if(i==0)
    for(;i<sizeof(Hs)/sizeof(double);i++)
      Hs[i]=log(Hs[i]);
  Yh=InterpolateTable(Hs,sH,sizeof(Hs)/sizeof(double),log(H));
  Yw=InterpolateTable(Ws,sW,sizeof(Ws)/sizeof(double),W);
  Yt=Yh+(Yw-Yh)*(Xt-Xh)/(Xw-Xh);
  return(InterpolateTable(sT,Ts,sizeof(Ts)/sizeof(double),Yt));
  }

double PondTemp(double Twb,double mph) /* Langhaar's temperature from wet-bulb */
  {
  return(max(32,max(Twb+1,Tequi((Rheat(Twb,mph)+solar),mph))));
  }

double RequiredPondArea(double Twb,double mph,double Thw,double Tcw)
  {
  double dT1,dT2,P,Q,Te;
  if(Tcw<=Twb)
    return(999.);
  if(Thw<=Tcw)
    return(0.);
  Te=PondTemp(Twb,mph);
  dT1=Thw-Te;
  dT2=Tcw-Te;
  if(dT2<=0.)
    return(999.);
  P=Pfactor(dT1,dT2);
  Q=Qfactor(Te,mph);
  return(P*Q);
  }

/*******************************************************************************
  PLANT SIMULATION SPECIFIC OBJECTS
*******************************************************************************/

#define Trated 3425. /* 100% rated thermal power */
#define Erated 1175. /* 100% rated electrical power */
int gauge=20;        /* condenser tube gauge */
int Nplug=0;         /* number of plugged condenser tubes */
int Npump=4;         /* number of CCW pumps */
int Ntube=44764;     /* number of condenser tubes */
double baro=14.696;  /* barometric pressure [psia] */
double bp;           /* condenser pressure/turbine backpressure [in.HgA] */
double bpmax=5.5;    /* maximum turbine backpressure [in.HgA] */
double Cc=0.95;      /* condenser cleanliness factor */
double Cm=0.94;      /* condenser tube material factor */
double Di;           /* condenser tube inside diameter [inch] */
double Do=0.875;     /* condenser tube outside diameter [inch] */
double heat=Trated;  /* reactor heat input [MWt] */
double Hstatic=6.163;/* static head across CCW loop */
double Hvform=12.17; /* form drag-like (V) drop across CCW loop */
double load=1200.;   /* net generator output [MWe] */
double Ltube=91.73;  /* condenser tube length (total) [ft] */
double Qc=410000.;   /* condenser cooling water flow rate [gpm] */
double rh=0.6;       /* ambient relative humidity */
double Tcold;        /* condenser inlet temperature [F] */
double Tdb=59.;      /* ambient dry-bulb temperature [F] */
double Thot;         /* condenser cooling water temperatures [F] */
double Twb=55.;      /* ambient wet-bulb temperature [F] */

/* standard tubing wall thickness for gauges 12-24 */
double Wt[13]={0.109,0.095,0.083,0.072,0.065,0.058,0.049,
               0.042,0.035,0.032,0.028,0.025,0.022};

double HeatrateCorrectionFactor(double pl,double bp)
  {
  double hrcf,Ts;
  Ts=fTsat(bp*0.49115);
  hrcf=((-0.0337506*pl+(0.762123*Ts-76.9946))*pl+
    ((-0.00484629*Ts-0.641417)*Ts+114.355))*pl
    +((2.75779E-5*Ts-0.000100014)*Ts+0.12553)*Ts-40.1616;
  return(hrcf);
  }

double LoadCorrectionFactor(double pl,double bp)
  {
  double cf;
  cf=HeatrateCorrectionFactor(pl,bp);
  return(-100.*cf/(100.+cf));
  }

double GeneratorOutput(double heat,double bp)
  {
  double cf,gen,pl;
  pl=heat/Trated;
  gen=Erated*((-0.10336*pl+1.256)*pl-0.15264);
  cf=LoadCorrectionFactor(pl,bp);
  return(gen*(1.+cf/100.));
  }

double CondenserHeatRejection(double heat,double bp)
  {
  double go,qr;
  go=GeneratorOutput(heat,bp);
  qr=heat-go;
  return(0.98*qr*3412141.633);
  }

double CondenserRise(double heat,double bp,double gpm)
  {
  return(CondenserHeatRejection(heat,bp)/gpm/500.);
  }

double WaterTemperatureCorrectionFactor(double T)
  {
  return(0.376769/((((2.15973E-9*T-1.13681E-6)*T+0.000221828)*T-0.0195586)*T+1));
  }

void FindBackpressure(double heat,double gpm,double Tcw,double*bp,double*Thw,double*MWe)
  {
  int iter;
  double area,bp1,bp2,Ct,Cv,effectiveness,rise,Tsat,NTU,velocity;
  if(heat<100.||gpm<10000.)
    {
    *Thw=Tcw;
    *bp=0.;
    *MWe=0.;
    return;
    }
  Di=Do-2.*Wt[gauge-12];
  Ct=0.107*Do/Di/Di;
  Di/=12.;
  area=Ntube*M_PI*Di*Di/4.;
  velocity=gpm/area/500.;
  Cv=263./sqrt(velocity);
  NTU=Ct*Cv*Cm*Cc*Ltube/500.;
  effectiveness=exp(-NTU*WaterTemperatureCorrectionFactor(Tcw));
  *bp=fPsat(Tcw)/0.49115;
  for(iter=0;iter<16;iter++)
    {
    rise=CondenserRise(heat,*bp,gpm);
    *Thw=Tcw+rise;
    Tsat=*Thw+rise*effectiveness/(1.-effectiveness);
    bp1=*bp;
    bp2=fPsat(Tsat)/0.49115;
    *bp=sqrt(bp1*bp2);
    if(iter>2&&fabs(bp1-bp2)<0.001)
      break;
    }
  *MWe=GeneratorOutput(heat,*bp);
  }

double PumpCurve(double gpm)
  {
  double x;
  x=gpm/247000.;
  return(45.*((((1.64822*x-6.75411)*x+8.2608)*x-4.28862)*x+2.13944));
  }

double WoodsFrictionFactor(double Re,double k)
  {
  double a,b,c;
  a=0.094*pow(k,0.225)+0.53*k;
  b=88*pow(k,0.44);
  c=1.62*pow(k,0.134);
  return(a+b/pow(Re,c));
  }

#define gravity 32.174

double CondenserFlow(int pumps)
  {
  int iter;
  double area,drop,friction,gpm,gpm1,gpm2,head,Reynolds,V;
  double viscosity=0.0000122; /* kinematic viscosity [ft/sec] of water @ 60F */
  double roughness=0.00006; /* relative roughness of condenser tubes */
  if(pumps<1)
    return(0);
  Di=Do-2.*Wt[gauge-12];
  area=Ntube*M_PI*pow(Di/12.,2)/4.;
  gpm1=1000.;
  gpm2=pumps*500000.;
  for(iter=0;iter<16;iter++)
    {
    gpm=(gpm1+gpm2)/2.;
    V=gpm/area/500.;
    Reynolds=V*(Di/12.)/viscosity;
    friction=WoodsFrictionFactor(Reynolds,roughness);
    drop=Hstatic+Hvform*pow(gpm/729350.,2)
      +friction*(Ltube/(Di/12.))*V*V/2./gravity;
    head=PumpCurve(gpm/pumps);
    if(drop<head)
      gpm1=gpm;
    else
      gpm2=gpm;
    }
  return(gpm);
  }

double FindLoad(
  double heat,/* heat=reactor power [MWt] */
  double gpm, /* condenser flow [gpm] */
  double Twb, /* wet-bulb temperature [F] */
  double*Tcw, /* condenser inlet water temperature [F] */
  double*Thw, /* condenser exit water temperature [F] */
  double*bp)  /* backpressure [in.HgA] */
  {
  int iter;
  double Apond,load,Tc1,Tc2;
  Apond=PondArea(Epond);
  Tc1=Twb;
  Tc2=150.;
  for(iter=0;iter<32;iter++)
    {
    *Tcw=(Tc1+Tc2)/2.;
    FindBackpressure(heat,gpm,*Tcw,bp,Thw,&load);
    if(RequiredPondArea(Twb,Vwind,*Thw,*Tcw)*gpm/43560.>Apond)
      Tc1=*Tcw;
    else
      Tc2=*Tcw;
    }
  return(load);
  }

void printc(char*format,...)
  {
  char bufr[128],*ptr;
  DWORD l;
  va_list arg_marker;
  if(!stdOutput)
    return;
  va_start(arg_marker,format);
  if(vsprintf(bufr,format,arg_marker)>=sizeof(bufr))
    FatalAppExit(0,"printc(): buffer overflow");
  if((ptr=strchr(bufr,'\n'))!=NULL)
    strcpy(ptr,"\r\n");
  l=(DWORD)strlen(bufr);
  WriteConsole(stdOutput,bufr,l,&l,NULL);
  }

void Simulation()
  {
  int bisect;
  double hmin,hmax;
  Qc=CondenserFlow(Npump);
  hmin=0.;
  hmax=2.*heat;
  for(bisect=0;bisect<32;bisect++)
    {
    heat=(hmin+hmax)/2.;
    load=FindLoad(heat,Qc,Twb,&Tcold,&Thot,&bp);
    if(bp<bpmax)
      {
      if(bisect==0)
        break;
      hmin=heat;
      }
    else
      hmax=heat;
    }
  printc("%3.0lf %4.2lf %4.0lf %4.0lf %5.1lf %5.1lf %4.1lf %4.2lf\n",Tdb,rh,heat,load,Tcold,Thot,Thot-Tcold,bp);
  }

typedef struct{
  double min;
  double max;
  int pts;
  }VAR;

void CreateReactorPlot()
  {
  int i,j;
  double bpj,ht[4];
  VAR BP={1.5,5.0,21};
  plot.Title ="Impact of Backpressure on Capacity";
  plot.Xlabel="Backpressure [in.HgA]";
  plot.Ylabel="Generator Output [MWe]";
  DeallocateCurves();
  plot.curves=4;
  plot.points=calloc(plot.curves,sizeof(int));
  plot.points[0]=
  plot.points[1]=
  plot.points[2]=
  plot.points[3]=BP.pts;
  plot.legend=2;
  plot.label=calloc(plot.curves,sizeof(char*));
  for(j=0;j<plot.curves;j++)
    {
    ht[j]=(100-10*j)*Trated/100.;
    sprintf(cbuf,"%4.0lf MWt",ht[j]);
    plot.label[j]=malloc(strlen(cbuf)+1);
    strcpy(plot.label[j],cbuf);
    }
  plot.Xp=calloc(plot.curves*BP.pts,sizeof(double));
  plot.Yp=calloc(plot.curves*BP.pts,sizeof(double));
  for(i=0;i<BP.pts;i++)
    {
    bpj=BP.min+i*(BP.max-BP.min)/(BP.pts-1);
    for(j=0;j<plot.curves;j++)
      {
      plot.Xp[BP.pts*j+i]=bpj;
      plot.Yp[BP.pts*j+i]=GeneratorOutput(ht[j],bpj);
      }
    }
  }

void CreateTurbinePlot()
  {
  int i,j;
  double bp;
  VAR BP={0.5,5.5,31};
  plot.Title ="Impact of Backpressure on Heat Rate";
  plot.Xlabel="Backpressure [in.HgA]";
  plot.Ylabel="Heat Rate Correction [%]";
  DeallocateCurves();
  plot.curves=4;
  plot.points=calloc(plot.curves,sizeof(int));
  plot.points[0]=
  plot.points[1]=
  plot.points[2]=
  plot.points[3]=BP.pts;
  plot.legend=0;
  plot.label=calloc(plot.curves,sizeof(char*));
  for(j=0;j<4;j++)
    {
    sprintf(cbuf,"%3i%% steamflow",(4-j)*25);
    plot.label[j]=malloc(strlen(cbuf)+1);
    strcpy(plot.label[j],cbuf);
    }
  plot.Xp=calloc(plot.curves*BP.pts,sizeof(double));
  plot.Yp=calloc(plot.curves*BP.pts,sizeof(double));
  for(i=0;i<BP.pts;i++)
    {
    bp=BP.min+i*(BP.max-BP.min)/(BP.pts-1);
    for(j=0;j<plot.curves;j++)
      {
      plot.Xp[BP.pts*j+i]=bp;
      plot.Yp[BP.pts*j+i]=HeatrateCorrectionFactor((4-j)/4.,bp);
      }
    }
  }

void CreatePondPlot()
  {
  int c,i,p;
  double Wb,Vw;
  VAR WB={80,30, 6};
  VAR VW={ 0,30,20};
  plot.Title ="Equilibrium Pond Temperature vs. Wet-Bulb";
  plot.Xlabel="Wind Speed [mph]";
  plot.Ylabel="Equilibrium Pond Temperature [F]";
  DeallocateCurves();
  plot.curves=WB.pts;
  plot.legend=1;
  plot.points=calloc(WB.pts,sizeof(int));
  plot.label=calloc(WB.pts,sizeof(char*));
  plot.Xp=calloc(WB.pts*VW.pts,sizeof(double));
  plot.Yp=calloc(WB.pts*VW.pts,sizeof(double));
  for(c=i=0;c<WB.pts;c++)
    {
    Wb=WB.min+c*(WB.max-WB.min)/(WB.pts-1);
    plot.points[c]=VW.pts;
    sprintf(cbuf,"Twb=%1.0lfF",Wb);
    plot.label[c]=calloc(strlen(cbuf)+1,1);
    strcpy(plot.label[c],cbuf);
    for(p=0;p<VW.pts;p++,i++)
      {
      Vw=VW.min+p*(VW.max-VW.min)/(VW.pts-1);
      plot.Xp[i]=Vw;
      plot.Yp[i]=PondTemp(Wb,Vw);
      }
    }
  }

void CreateCondenserPlot()
  {
  int i,j;
  double bp,ht[4],load,Tin,Thw;
  VAR TIN={35.,115.,17};
  plot.Title ="Condenser Performance";
  plot.Xlabel="Cooling Water Inlet Temperature [F]";
  plot.Ylabel="Backpressure [in.HgA]";
  DeallocateCurves();
  plot.curves=4;
  plot.points=calloc(plot.curves,sizeof(int));
  plot.points[0]=
  plot.points[1]=
  plot.points[2]=
  plot.points[3]=TIN.pts;
  plot.legend=0;
  plot.label=calloc(plot.curves,sizeof(char*));
  for(j=0;j<plot.curves;j++)
    {
    ht[j]=(100-10*j)*Trated/100.;
    sprintf(cbuf,"%4.0lf MWt",ht[j]);
    plot.label[j]=malloc(strlen(cbuf)+1);
    strcpy(plot.label[j],cbuf);
    }
  plot.Xp=calloc(plot.curves*TIN.pts,sizeof(double));
  plot.Yp=calloc(plot.curves*TIN.pts,sizeof(double));
  for(i=0;i<TIN.pts;i++)
    {
    Tin=TIN.min+i*(TIN.max-TIN.min)/(TIN.pts-1);
    for(j=0;j<plot.curves;j++)
      {
      FindBackpressure(ht[j],Qc,Tin,&bp,&Thw,&load);
      plot.Xp[TIN.pts*j+i]=Tin;
      plot.Yp[TIN.pts*j+i]=bp;
      }
    }
  }

/*******************************************************************************
  WINDOWS GUI OBJECTS
*******************************************************************************/

int WINAPI PushProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_LBUTTONDBLCLK)
    {
    PostMessage(hWnd,WM_LBUTTONDOWN,wParam,lParam);
    PostMessage(hWnd,WM_LBUTTONUP,wParam,lParam);
    }
  if(wMsg==WM_LBUTTONDOWN)
    {
    SetWindowLong(hWnd,GWL_ID,TRUE);
    InvalidateRect(hWnd,NULL,TRUE);
    UpdateWindow(hWnd);
    return(FALSE);
    }
  if(wMsg==WM_LBUTTONUP)
    {
    SetWindowLong(hWnd,GWL_ID,FALSE);
    InvalidateRect(hWnd,NULL,TRUE);
    UpdateWindow(hWnd);
    display_plot^=1;
    if(display_plot)
      {
      ShowWindow(pCondenser,SW_HIDE);
      ShowWindow(pTurbine  ,SW_HIDE);
      ShowWindow(pPond     ,SW_HIDE);
      ShowWindow(pReactor  ,SW_HIDE);
      ShowWindow(hWnd      ,SW_SHOW);
      if(hWnd==pCondenser)
        CreateCondenserPlot();
      else if(hWnd==pTurbine)
        CreateTurbinePlot();
      else if(hWnd==pPond)
        CreatePondPlot();
      else if(hWnd==pReactor)
        CreateReactorPlot();
      }
    else
      {
      ShowWindow(pCondenser,SW_SHOW);
      ShowWindow(pPond     ,SW_SHOW);
      ShowWindow(pTurbine  ,SW_SHOW);
      ShowWindow(pReactor  ,SW_SHOW);
      }
    InvalidateRect(hPlot,NULL,TRUE);
    return(FALSE);
    }
  if(wMsg==WM_PAINT)
    {
    int i,x,y;
    HDC hDC;
    HPEN hPenO,hPenS,hPenW;
    PAINTSTRUCT pS;
    POINT P;
    RECT cR;
    static SIZE sText;
    if((hDC=BeginPaint(hWnd,&pS))==NULL)
      Abort(__LINE__,"PushProc(): can't get device context\nWin Err code %i",GetLastError());
    GetClientRect(hWnd,&cR);
    for(i=0;i<2;i++)
      {
      FrameRect(hDC,&cR,GetStockObject(BLACK_BRUSH));
      cR.top++;
      cR.left++;
      cR.right--;
      cR.bottom--;
      }
    if(GetWindowLong(hWnd,GWL_ID))
      {
      hPenW=CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNSHADOW));
      hPenS=CreatePen(PS_SOLID,1,GetSysColor(COLOR_WINDOW));
      }
    else
      {
      hPenW=CreatePen(PS_SOLID,1,GetSysColor(COLOR_WINDOW));
      hPenS=CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNSHADOW));
      }
    for(i=0;i<3;i++)
      {
      hPenO=SelectObject(hDC,hPenW);
      MoveToEx(hDC,cR.left,cR.top,&P);
      LineTo(hDC,cR.right,cR.top);
      MoveToEx(hDC,cR.left,cR.top,&P);
      LineTo(hDC,cR.left,cR.bottom);
      hPenW=SelectObject(hDC,hPenS);
      MoveToEx(hDC,cR.left,cR.bottom-1,&P);
      LineTo(hDC,cR.right,cR.bottom-1);
      MoveToEx(hDC,cR.right-1,cR.top,&P);
      LineTo(hDC,cR.right-1,cR.bottom);
      hPenS=SelectObject(hDC,hPenO);
      cR.top++;
      cR.left++;
      cR.right--;
      cR.bottom--;
      }
    DeleteObject(hPenS);
    DeleteObject(hPenW);
    PaintBitmap(hDC,&cR,(BITMAPINFOHEADER*)GetWindowLong(hWnd,GWL_USERDATA),1);
    SetBkMode(hDC,TRANSPARENT);
    GetWindowText(hWnd,cbuf,sizeof(cbuf));
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    x=(cR.right+cR.left-sText.cx)/2;
    y=cR.bottom-sText.cy-2;
    SetTextColor(hDC,RGB(0xFF,0xFF,0xFF));
    TextOut(hDC,x+1,y-1,cbuf,strlen(cbuf));
    TextOut(hDC,x+1,y+1,cbuf,strlen(cbuf));
    TextOut(hDC,x-1,y+1,cbuf,strlen(cbuf));
    TextOut(hDC,x-1,y-1,cbuf,strlen(cbuf));
    SetTextColor(hDC,RGB(0x00,0x00,0x00));
    TextOut(hDC,x,y,cbuf,strlen(cbuf));
    EndPaint(hWnd,&pS);
    return(FALSE);
    }
  return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  }

int WINAPI MainProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_CLOSE)
    {
    PostQuitMessage(0);
    return(FALSE);
    }
  if(wMsg==WM_COMMAND)
    {
    if(HIWORD(wParam)==BN_CLICKED)
      {
      if(LOWORD(wParam)==PUSH_ABOUT)
        DialogBox(hInst,"ABOUT",hMain,(DLGPROC)AboutProc);
      else if(LOWORD(wParam)==PUSH_EXIT)
        PostQuitMessage(0);
      else if(LOWORD(wParam)==PUSH_HELP)
        WinHelp(hMain,"Braidwood.hlp",HELP_CONTENTS,0);
      else if(LOWORD(wParam)==PUSH_MAIN)
        PostMessage(pReactor,WM_LBUTTONDBLCLK,0,0);
      return(FALSE);
      }
    return(DefWindowProc(hWnd,wMsg,wParam,lParam));
    }
  if(wMsg==WM_CREATE)
    return(FALSE);
  if(wMsg==WM_CTLCOLORDLG)
    return((DWORD)GetStockObject(LTGRAY_BRUSH));
  if(wMsg==WM_DESTROY)
    return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  if(wMsg==WM_PAINT)
    {
    HDC hDC;
    PAINTSTRUCT pS;
    if((hDC=BeginPaint(hWnd,&pS))==NULL)
      Abort(__LINE__,"can't get device context\nWin Err code %i",GetLastError());
    EndPaint(hWnd,&pS);
    return(FALSE);
    }
  if(wMsg==WM_SIZE)
    {
    PositionWindows();
    return(FALSE);
    }
  if(wMsg==WM_TIMER)
    return(FALSE);
  return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  }

void CreateFonts()
  {
  fBold      =CreateStandardFont("Arial",FW_HEAVY , 0);
  fHorizontal=CreateStandardFont("Arial",FW_MEDIUM, 0);
  fVertical  =CreateStandardFont("Arial",FW_MEDIUM,90);
  }

void*GetResource(char*rName,char*type)
  {
  void*rLock;
  HGLOBAL rLoad;
  HRSRC rFind;
  if((rFind=FindResource(hInst,rName,type))==NULL)
    Abort(__LINE__,"can't find resource %s\nWin Err code %i",rName,GetLastError());
  if((rLoad=LoadResource(hInst,rFind))==NULL)
    Abort(__LINE__,"can't load resource %s\nWin Err code %i",rName,GetLastError());
  if((rLock=LockResource(rLoad))==NULL)
    Abort(__LINE__,"can't lock resource %s\nWin Err code %i",rName,GetLastError());
  return(rLock);
  }

BITMAPINFOHEADER*GetBitmap(char*name)
  {
  return((BITMAPINFOHEADER*)((BYTE*)GetResource(name,RT_BITMAP)));
  }

void LoadBitmaps()
  {
  bCondenser=GetBitmap("CONDENSER");
  bPlant    =GetBitmap("PLANT");
  bPond     =GetBitmap("POND");
  bReactor  =GetBitmap("REACTOR");
  bTurbine  =GetBitmap("TURBINE");
  }

void CreateWindows()
  {
  sizes.screen.x=GetSystemMetrics(SM_CXSCREEN);
  sizes.screen.y=GetSystemMetrics(SM_CYSCREEN);
  sizes.plant.x=bPlant->biWidth;
  sizes.plant.y=bPlant->biHeight;
  sizes.frame.x=GetSystemMetrics(SM_CXFRAME)/2;
  sizes.frame.y=GetSystemMetrics(SM_CYFRAME)/2;
  sizes.button.x=GetSystemMetrics(SM_CXMINTRACK)+2*sizes.frame.x;
  sizes.button.y=GetSystemMetrics(SM_CYVTHUMB)+2*sizes.frame.y;
  sizes.push.x=sizes.button.x;
  sizes.push.y=3*(sizes.push.x-10)/4+10;
  sizes.client.x=sizes.plant.x+sizes.push.x+sizes.frame.x+8;
  sizes.client.y=sizes.plant.y+8;
  sizes.window.x=sizes.client.x+2*sizes.frame.x;
  sizes.window.y=sizes.client.y+2*sizes.frame.x+GetSystemMetrics(SM_CYCAPTION);
  if((hMain=CreateWindow("MAIN","Braidwood Nuclear Heat Rejection System Model",
      WS_OVERLAPPEDWINDOW|WS_VISIBLE,
      (sizes.screen.x-sizes.window.x)/2,
      (sizes.screen.y-sizes.window.y)/2,
      sizes.window.x,sizes.window.y,
      NULL,NULL,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create main window\nWin Err code %i",GetLastError());
  if((hPlot=CreateWindow("DRAW",NULL,
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create draw window\nWin Err code %i",GetLastError());
  if((pReactor=CreateWindow("PUSH","Reactor",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create reactor button\nWin Err code %i",GetLastError());
  SetWindowLong(pReactor,GWL_USERDATA,(long)bReactor);
  if((pTurbine=CreateWindow("PUSH","Steam System",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create turbine button\nWin Err code %i",GetLastError());
  SetWindowLong(pTurbine,GWL_USERDATA,(long)bTurbine);
  if((pPond=CreateWindow("PUSH","Cooling Pond",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create pond button\nWin Err code %i",GetLastError());
  SetWindowLong(pPond,GWL_USERDATA,(long)bPond);
  if((pCondenser=CreateWindow("PUSH","Condenser",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create Condenser button\nWin Err code %i",GetLastError());
  SetWindowLong(pCondenser,GWL_USERDATA,(long)bCondenser);
  if((bExit=CreateWindow("BUTTON","Exit",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|BS_PUSHBUTTON,
      0,0,0,0,hMain,(void*)PUSH_EXIT,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create exit button\nWin Err code %i",GetLastError());
  if((bAbout=CreateWindow("BUTTON","About",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|BS_PUSHBUTTON,
      0,0,0,0,hMain,(void*)PUSH_ABOUT,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create about button\nWin Err code %i",GetLastError());
  if((bHelp=CreateWindow("BUTTON","Help",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|BS_PUSHBUTTON,
      0,0,0,0,hMain,(void*)PUSH_HELP,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create help button\nWin Err code %i",GetLastError());
  PositionWindows();
  }

void Register(char*cName,void*wProc,HBRUSH hBrush,char*cursor)
  {
  WNDCLASS wc;
  memset(&wc,0,sizeof(WNDCLASS));
  wc.hInstance    =hInst;
  wc.hIcon        =LoadIcon(hInst,"ICON");
  wc.style        =CS_HREDRAW|CS_VREDRAW;
  wc.lpszClassName=cName;
  wc.lpfnWndProc  =(WNDPROC)wProc;
  wc.hbrBackground=hBrush;
  wc.hCursor      =LoadCursor(NULL,cursor);
  if(!RegisterClass(&wc))
    Abort(__LINE__,"can't register [%s] class\nWin Err code %i",cName,GetLastError());
  }

void RegisterClasses()
  {
  Register("MAIN",MainProc,GetStockObject(LTGRAY_BRUSH),IDC_ARROW);
  Register("DRAW",DrawProc,GetStockObject(LTGRAY_BRUSH),IDC_NO   );
  Register("PUSH",PushProc,GetStockObject(DKGRAY_BRUSH),IDC_ARROW);
  }

void Tables()
  {
  double area,gpm,hours,ht,volume;
  printc("pumps gpm\n");
  for(Npump=1;Npump<=4;Npump++)
    {
    gpm=CondenserFlow(Npump);
    printc("  %i %6.0lf\n",Npump,gpm);
    }
  printc("Tdb  RH   MWt  MWe  Tcw   Thw  rise  bp\n");
  Tdb=59.;
  rh=0.6;
  Twb=fTwdbp(14.696,Tdb,rh);
  for(ht=2400.;ht<3800.1;ht+=100.)
    {
    heat=ht;
    Simulation();
    }
  printc("Tdb  RH   MWt  MWe  Tcw   Thw  rise  bp\n");
  for(Tdb=30.;Tdb<110.1;Tdb+=10.)
    {
    for(rh=0.2;rh<1.01;rh+=0.2)
      {
      Twb=fTwdrh(14.696,Tdb,rh);
      heat=Trated;
      Simulation();
      }
    }
  printc("cooling pond geometry\n");
  printc("feet acre acre-ft hours\n");
  for(Epond=584.;Epond<600.1;Epond+=1.)
    {
    area=PondArea(Epond);
    volume=PondVolume(Epond);
    hours=volume*43560./(gpm*500.);
    printc("%4.0lf %4.0lf %5.0lf %4.1lf\n",Epond,area,volume,hours);
    }
  }

void BatchMode()
  {
  int da,hr,i,mo,yr;
  double baro,ghi,knots,Tdb,Tdp,Tpond;
  FILE*fi,*fo;
  if(!AllocConsole())
    Abort(__LINE__,"can't create console\nWin Err code %i",GetLastError());
  CenterWindowOnScreen(GetConsoleWindow());
  BringWindowToTop(GetConsoleWindow());
  stdOutput=GetStdHandle(STD_OUTPUT_HANDLE);
  stdInput=GetStdHandle(STD_INPUT_HANDLE);
  if((fo=fopen("Braidwood.csv","wt"))==NULL)
    Abort(__LINE__,"can't create output file");
  Tables();
  if((fi=fopen(__argv[1],"rt"))==NULL)
    Abort(__LINE__,"can't open input file [%s]",__argv[1]);
  fprintf(fo,"Braidwood Nuclear Plant Simulation\n");
  fprintf(fo,"mm/dd/yyyy,heat,load,bp,Tcw,Thw,Tpond\n");
  fprintf(fo,"hh:mm,MWt,MWe,in.HgA,F,F,F\n");
  i=0;
  while(fgets(cbuf,sizeof(cbuf),fi))
    {
    if(sscanf(cbuf,"%d/%d/%d %d:%*d,%lf,%lf,%lf,%lf,%lf",&mo,&da,&yr,&hr,&baro,&Tdb,&Tdp,&knots,&ghi)==9)
      {
      Twb=fTwdbp(baro/68.94757293,Tdb*1.8+32.,Tdp*1.8+32.);
      Vwind=knots*1.15078;
      solar=300.+ghi/300.;
      heat=Trated;
      Simulation();
      i++;
      if(i==1)
        Tpond=Thot;
      else if(i==2)
        Tpond=(Tpond+Thot)/2.;
      else if(i==3)
        Tpond=(2.*Tpond+Thot)/3.;
      else
        Tpond=(3.*Tpond+Thot)/4.;
      fprintf(fo,"%d/%d/%d %d:00,%.1lf,%.1lf,%.1lf,%.3lf,%.1lf,%.1lf\n",mo,da,yr,hr,heat,load,bp,Tcold,Thot,Tpond);
      }
    }
  fclose(fi);
  fclose(fo);
  BringWindowToTop(GetConsoleWindow());
  MessageBox(NULL,"click OK to exit program","batch run complete",MB_SYSTEMMODAL|MB_ICONSTOP|MB_OK);
  FreeConsole();
  }

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevious,char*lpszLine,int nShow)
  {
  MSG msg;
  hInst=hInstance;
  if(__argc>1)
    {
    BatchMode();
    ExitProcess(0);
    }
  LoadBitmaps();
  CreateFonts();
  RegisterClasses();
  CreateWindows();
  while(GetMessage(&msg,NULL,0,0))
    {
    if(msg.message==WM_SYSKEYDOWN&&msg.wParam==VK_F4)
      break;
    if(!TranslateMessage(&msg))
      DispatchMessage(&msg);
    }
  return(0);
  }
