#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <io.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define DEFLATED 8
#define OK 0
extern int decompress;
extern int ifd;
extern int method;
extern int no_name;
extern int no_time;
extern int ofd;
extern int quiet;
extern int verbose;
extern long ifile_size;
extern time_t time_stamp;
int get_method(int);
int unzip(int,int);
extern void clear_bufs();

struct{
  char name1[100];
  char mode[8];
  char owner[8];
  char group1[8];
  char fsize[12];
  char time[12];
  char checksum[8];
  char flag;
  char name2[100];
  char ustar[6];
  char version[2];
  char user[32];
  char group2[32];
  char major[8];
  char minor[8];
  char name3[155];
  char padding[12];
  }header;

void Abort(int code,char*format,...)
  {
  va_list arg_marker;
  fprintf(stderr,"\nFatal Error Code: %i\n",code);
  va_start(arg_marker,format);
  vfprintf(stderr,format,arg_marker);
  fprintf(stderr,"\n");
  _fcloseall();
  exit(1);
  }

void CleanUp(char*bufr)
  {
  char date[8],*ptr;
  if(strchr("0123456789",bufr[0]))
    {
    memcpy(date,bufr+14,8);
    memmove(bufr+16,bufr+14,126);
    bufr[14]=date[4];
    bufr[15]=date[5];
    bufr[16]='/';
    bufr[17]=date[6];
    bufr[18]=date[7];
    bufr[19]='/';
    bufr[20]=date[0];
    bufr[21]=date[1];
    bufr[22]=date[2];
    bufr[23]=date[3];
    while((ptr=strstr(bufr,"* "))!=NULL)
      strcpy(ptr,ptr+1);
    while((ptr=strpbrk(bufr,"ABCDEFGHIJKLMNOPQRSTUVWXYZ"))!=NULL)
      *ptr=' ';
    }
  while((ptr=strstr(bufr,"  "))!=NULL)
    strcpy(ptr,ptr+1);
  while((ptr=strchr(bufr,' '))!=NULL)
    *ptr=',';
  }

int main(int argc,char**argv,char**envp)
  {
  char*try="specify USAF and WBAN as in\n"
           "getstn 123456 12345\n";
  char archive[FILENAME_MAX],unzipped[21],zipped[21];
  char bufr[512],*ptr;
  int asize,fsize,i,l,year;
  static char USAF[7],WBAN[6],YEAR[5];
  FILE*Farc,*Finp,*Fout,*Fcsv;
  printf("GSOD station extractor\n");
  if(argc!=3)
    {
    printf("no station name\n%s",try);
    return(0);
    }
  if(strlen(argv[1])!=6||strspn(argv[1],"0123456789")<6)
    {
    printf("USAF must be 6 numbers\n%s",try);
    return(0);
    }
  if(strlen(argv[2])!=5||strspn(argv[2],"0123456789")<5)
    {
    printf("WBAN must be 5 numbers\n%s",try);
    return(0);
    }
  printf("GSOD data coverage for %s:%s\n",argv[1],argv[2]);
  sprintf(archive,"%s-%s.csv",argv[1],argv[2]);
  Fcsv=fopen(archive,"wt");
  for(year=1900;year<2099;year++)
    {
    sprintf(archive,"gsod_%i.tar",year);
    if((Farc=fopen(archive,"rb"))==NULL)
      continue;
    printf("%s\n",archive);
    asize=_filelength(_fileno(Farc));
    while(fread(&header,1,sizeof(header),Farc)==sizeof(header))
      {
      if(_memicmp(header.ustar,"ustar",5))
        continue;
      if(header.flag!='0')
        continue;
      for(l=1,fsize=i=0;i<11;i++,l*=8)
        fsize+=l*(int)(header.fsize[10-i]-'0');
      if(asize-ftell(Farc)<512*((fsize+511)/512))
        break;
      if((ptr=strrchr(header.name1,'/'))!=NULL)
        ptr++;
      else
        ptr=header.name1;
      memcpy(USAF,ptr,6);
      if(strspn(USAF,"0123456789")<6)
        Abort(__LINE__,"the USAF number must consist entirely of digits 0-9\n%s",ptr);
      if(ptr[6]!='-')
        Abort(__LINE__,"expected filename to be 123456-12345-1234 but found\n%s",ptr);
      memcpy(WBAN,ptr+7,5);
      if(strspn(WBAN,"0123456789")<5)
        Abort(__LINE__,"the WBAN number must consist entirely of digits 0-9\n%s",ptr);
      if(ptr[12]!='-')
        Abort(__LINE__,"expected filename to be 123456-12345-1234 but found\n%s",ptr);
      memcpy(YEAR,ptr+13,4);
      if(strspn(YEAR,"0123456789")<4)
        Abort(__LINE__,"the YEAR number must consist entirely of digits 0-9\n%s",ptr);
      if(atoi(YEAR)!=year)
        Abort(__LINE__,"expected year to be %i but found\n%s",year,ptr);
      if(strcmp(USAF,argv[1])!=0||strcmp(WBAN,argv[2])!=0)
        {
        while(fsize>0)
          {
          if((l=fread(bufr,1,512,Farc))!=512)
            Abort(__LINE__,"an error occurred while reading archive\n512 bytes expected %i bytes read",l);
          fsize-=min(512,fsize);
          }
        continue;
        }
      sprintf(zipped,"%s-%s-%i.gz",USAF,WBAN,year);
      printf("extracting file %s %5i",zipped,fsize);
      if((Fout=fopen(zipped,"wb"))==NULL)
        Abort(__LINE__,"can't create extract file");
      while(fsize>0)
        {
        if((l=fread(bufr,1,512,Farc))!=512)
          Abort(__LINE__,"an error occurred while reading archive\n512 bytes expected %i bytes read",l);
        if((l=fwrite(bufr,1,min(512,fsize),Fout))!=min(512,fsize))
          Abort(__LINE__,"an error occurred while writing file\n%i bytes expected %i bytes written",min(512,fsize),l);
        fsize-=min(512,fsize);
        }
      fclose(Fout);
      if((Finp=fopen(zipped,"rb"))==NULL)
        Abort(__LINE__,"can't open extract file");
      ifd=_fileno(Finp);
      ifile_size=_filelength(ifd);
      sprintf(unzipped,"%s-%s-%i.op",USAF,WBAN,year);
      if((Fout=fopen(unzipped,"wb"))==NULL)
        Abort(__LINE__,"can't create decompressed file");
      ofd=_fileno(Fout);
      decompress=1;
      quiet=1;
      verbose=0;
      no_name=1;
      no_time=1;
      time_stamp=0;
      clear_bufs();
      method=get_method(ifd);
      if(method!=DEFLATED)
        Abort(__LINE__,"unsupported compression method");
      if(unzip(ifd,ofd)!=OK)
        Abort(__LINE__,"decompression failed");
      fclose(Fout);
      fclose(Finp);
      if((Finp=fopen(unzipped,"rt"))==NULL)
        Abort(__LINE__,"can't open decompressed file");
      while(fgets(bufr,sizeof(bufr),Finp))
        {
        CleanUp(bufr);
        fprintf(Fcsv,"%s",bufr);
        }
      fclose(Finp);
      fflush(Fcsv);
      remove(unzipped);
      remove(zipped);
      printf("\n");
      }
    fclose(Farc);
    }
  fclose(Fcsv);
  return(0);
  }
