#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#define _USE_MATH_DEFINES
#include <math.h>

double fPs(double Ts) /* saturation pressure (-160F to +212F) */
  {
  return(14.696*exp(((2.20235813606E-7*Ts-0.000140610815091)*Ts+0.0513198050511)*Ts-6.65862121962));
  }

double fRHdbp(double Tdb,double Tdp) /* RH from Tdb and Tdp */
  {
  if(Tdb<=Tdp)
    return(1.);
  return(fPs(Tdp)/fPs(Tdb));
  }

double sq(double x)
  {
  return x*x;
  }

typedef struct{int mo,da,yr;double Tavg,Tdp,Tmax,Tmin,baro;}DATA;

DATA*Data;
int ndays;

void ReadData(char*fname)
  {
  char bufr[256];
  int na,nb;
  double a,b;
  DATA data;
  FILE*fp;
  printf("reading data file: %s\n",fname);
  if((fp=fopen(fname,"rt"))==NULL)
    {
    fprintf(stderr,"can't open input file\n");
    exit(1);
    }
  ndays=0;
  while(fgets(bufr,sizeof(bufr),fp))
    if(sscanf(bufr,"%*d,%*d,%d/%d/%d,%lf,%*d,%lf,%*d,%lf,%d,%lf,%d,%*lf,%*d,%*lf,%*d,%*lf,%*lf,%lf,%lf",
        &data.mo,&data.da,&data.yr,&data.Tavg,&data.Tdp,&a,&na,&b,&nb,&data.Tmax,&data.Tmin)==11)
      ndays++;
  printf("  %i days found\n",ndays);
  if((Data=calloc(ndays,sizeof(DATA)))==NULL)
    {
    fprintf(stderr,"can't allocate memory\n");
    exit(1);
    }
  rewind(fp);
  ndays=0;
  while(fgets(bufr,sizeof(bufr),fp))
    {
    if(sscanf(bufr,"%*d,%*d,%d/%d/%d,%lf,%*d,%lf,%*d,%lf,%d,%lf,%d,%*lf,%*d,%*lf,%*d,%*lf,%*lf,%lf,%lf",
        &data.mo,&data.da,&data.yr,&data.Tavg,&data.Tdp,&a,&na,&b,&nb,&data.Tmax,&data.Tmin)==11)
      {
      if(na>nb)
        data.baro=a;
      else
        data.baro=b;
      data.baro*=0.01450377377;
      Data[ndays++]=data;
      }
    }
  fclose(fp);
  }

void WriteData(char*fname)
  {
  int hour,iday;
  double a,b1,b2,baro,T1,T2,Ta,Tb,Tdb,Tdp,Tx,rh;
  FILE*fp;
  printf("writing fixed data: %s\n",fname);
  if((fp=fopen(fname,"wt"))==NULL)
    {
    fprintf(stderr,"can't create output file\n");
    exit(1);
    }
  fprintf(fp,"m/d/y h:mm,baro,Tdb,Tdp,rh\n");
  for(a=iday=0;iday<ndays-1;iday++)
    {
    Ta=Data[iday].Tdp;
    Tb=Data[iday+1].Tdp;
    T1=Data[iday].Tmin;
    T2=Data[iday+1].Tmin;
    Tx=Data[iday].Tmax;
    b1=Data[iday].baro;
    b2=Data[iday+1].baro;
    for(hour=0;hour<24;hour++)
      {
      Tdb=T1+hour*(T2-T1)/24.+(Tx-T1-hour*(T2-T1)/24.)/exp(sq((hour-13.)/4.));
      Tdp=Ta+hour*(Tb-Ta)/24.;
      if(Tdp>Tdb)
        a=Tdp-Tdb;
      Tdp-=a;
      a/=M_SQRT2;
      baro=b1+hour*(b2-b1)/24.;
      rh=fRHdbp(Tdb,Tdp);
      fprintf(fp,"%i/%i/%i %02i:00,%.3lf,%.1lf,%.1lf,%.2lf\n",Data[iday].mo,
        Data[iday].da,Data[iday].yr,hour,baro,Tdb,Tdp,rh);
      }
    }
  fclose(fp);
  }

int main(int argc,char**argv,char**envp)
  {
  if(argc!=3)
    printf("no files specified... try\ntohourly infile.csv outfile.csv\n");
  else
    {
    ReadData(argv[1]);
    WriteData(argv[2]);
    }
  return(0);
  }
