/* Watts Bar Nuclear Plant
   Heat Rejection Model
   developed by
   dudley.benton@gmail.com
*/
#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "wattsbar.h"
#include "..\\moistair\\moistair.h"
#include "..\\steam\\steam.h"

extern int __argc;
extern char**__argv;
extern char**_environ;

void Abort(int code,char*format,...)
  {
  char cbuf[1024],head[32];
  va_list arg_marker;
  va_start(arg_marker,format);
  vsprintf(cbuf,format,arg_marker);
  sprintf(head,"Fatal Error Code %li",code);
  MessageBox(NULL,cbuf,head,MB_ICONSTOP|MB_OK);
  ExitProcess(0);
  }

void WindowText(HWND hWnd,char*format,...)
  {
  char cbuf[129];
  int l;
  va_list arg_marker;
  va_start(arg_marker,format);
  l=vsprintf(cbuf,format,arg_marker);
  if(l>=sizeof(cbuf))
    Abort(__LINE__,"WindowText(): buffer overflow");
  SetWindowText(hWnd,cbuf);
  }

void DrawLine(HDC hDC,DWORD rgb,int width,int x1,int y1,int x2,int y2)
  {
  HPEN hPen;
  POINT P;
  hPen=CreatePen(PS_SOLID,width,rgb);
  hPen=SelectObject(hDC,hPen);
  MoveToEx(hDC,x1,y1,&P);
  LineTo(hDC,x2,y2);
  hPen=SelectObject(hDC,hPen);
  DeleteObject(hPen);
  }

void GetTextSize(HDC hDC,char*string,int len,SIZE*S)
  {
  static TEXTMETRIC Tm;
  S->cx=S->cy=0;
  if(len>0)
    {
    GetTextExtentPoint32(hDC,string,len,S);
    if(S->cx<6||S->cy<8)
      {
      GetTextMetrics(hDC,&Tm);
      if(Tm.tmAveCharWidth<6||Tm.tmHeight<10)
        {
        S->cx=((2*len+1)*6)/2;
        S->cy=10;
        }
      else
        {
        S->cx=((2*len+1)*Tm.tmAveCharWidth)/2;
        S->cy=(9*Tm.tmHeight)/8;
        }
      }
    }
  }

HFONT CreateStandardFont(char*name,int weight,double angle)
  {
  HFONT hFont;
  LOGFONT*lFont;
  lFont=(LOGFONT*)calloc(sizeof(LOGFONT),1);
  strcpy(lFont->lfFaceName,name);
  lFont->lfWeight=weight;
  lFont->lfEscapement=(int)(angle*10.+0.5);
  hFont=CreateFontIndirect(lFont);
  free(lFont);
  return(hFont);
  }

void PaintBitmap(HDC hDC,RECT*rc,BITMAPINFOHEADER*bHead,int stretch)
  {
  int colors,h,H,l,t,w,W;
  BYTE*bImage;
  l=rc->left;
  t=rc->top;
  w=rc->right-rc->left;
  h=rc->bottom-rc->top;
  W=bHead->biWidth;
  H=bHead->biHeight;
  if(w<8||h<8||W<8||H<8)
    return;
  if(bHead->biBitCount==24)
    colors=0;
  else
    {
    colors=bHead->biClrUsed;
    if(colors==0)
      colors=1<<bHead->biBitCount;
    }
  bImage=((BYTE*)bHead)+sizeof(BITMAPINFOHEADER)+colors*sizeof(DWORD);
  if(stretch<0)
    SetDIBitsToDevice(hDC,l,t,W,H,0,0,0,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS);
  else if(stretch>0)
    StretchDIBits(hDC,l,t,w,h,0,0,W,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS,SRCCOPY);
  else if(w*abs(H)<abs(W)*h)
    StretchDIBits(hDC,l,t+(h-w*abs(H)/abs(W))/2,w,w*abs(H)/abs(W),0,0,W,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS,SRCCOPY);
  else
    StretchDIBits(hDC,l+(w-h*abs(W)/abs(H))/2,t,h*abs(W)/abs(H),h,0,0,W,H,bImage,(BITMAPINFO*)bHead,DIB_RGB_COLORS,SRCCOPY);
  }

int nint(double d)
  {
  if(d>0.)
    return((int)(d+0.5));
  if(d<0.)
    return((int)(d-0.5));
  return(0);
  }

/*******************************************************************************
  GLOBAL APPLICATION OBJECTS
*******************************************************************************/

HINSTANCE hInst;
HWND hMain;
HWND hPlot;
HWND bAbout;
HWND bExit;
HWND bHelp;
HWND pCondenser;
HWND pTower;
HWND pTurbine;
HWND pReactor;

HANDLE stdInput;
HANDLE stdOutput;

HFONT fBold;
HFONT fHorizontal;
HFONT fVertical;

BITMAPINFOHEADER*bCondenser;
BITMAPINFOHEADER*bPlant;
BITMAPINFOHEADER*bReactor;
BITMAPINFOHEADER*bTower;
BITMAPINFOHEADER*bTurbine;

char cbuf[1024];
int display_plot;

struct{
  POINT button;
  POINT client;
  POINT draw;
  POINT frame;
  POINT plant;
  POINT push;
  POINT screen;
  POINT window;
  }sizes;

DWORD color[16]={
  0x000000, /* black */
  0xA80000, /* blue */
  0x00A800, /* green */
  0xA8A800, /* cyan */
  0x0000A8, /* red */
  0xA800A8, /* magenta */
  0x00A8A8, /* brown */
  0xD0D0D0, /* white */
  0xA8A8A8, /* dark_gray */
  0xFC0000, /* light_blue */
  0x00FC00, /* light_green */
  0xFCFC00, /* light_cyan */
  0x0000FC, /* light_red */
  0xFC00FC, /* light_magenta */
  0x00FCFC, /* yellow */
  0xFCFCFC};/* bright_white */

struct{
  int    legend; /* legend location */
  int    curves; /* number of curves */
  char **label ; /* curve labels */
  int   *points; /* number of points along each curve */
  char  *Title ; /* plot title */
  char  *Xlabel; /* X axis label */
  double*Xp    ; /* X points */
  char  *Ylabel; /* Y axis label */
  double*Yp    ; /* Y points */
  }plot;

void PlotCurves(HDC hDC,RECT rWindow)
  {
  int c,i,p,x,x1,x2,y,y1,y2;
  double Ax,Ay,Bx,By,dX,dY,X,Xm,Xx,Y,Ym,Yx;
  HRGN hRgn;
  RECT rPlot;
  static SIZE sText,sTick,sXnum,sYnum;
  FillRect(hDC,&rWindow,GetStockObject(WHITE_BRUSH));
  for(i=0;i<2;i++)
    {
    FrameRect(hDC,&rWindow,GetStockObject(BLACK_BRUSH));
    rWindow.top++;
    rWindow.left++;
    rWindow.right--;
    rWindow.bottom--;
    }
  rWindow.left  +=2;
  rWindow.top   +=2;
  rWindow.right -=2;
  rWindow.bottom-=2;
  SelectClipRgn(hDC,NULL);
  SetBkMode(hDC,TRANSPARENT);
  SetTextColor(hDC,color[0]);
  SelectObject(hDC,fHorizontal);
  GetTextSize(hDC,plot.Title,strlen(plot.Title),&sText);
  TextOut(hDC,(rWindow.right-rWindow.left-sText.cx)/2,rWindow.top,plot.Title,strlen(plot.Title));
  SelectObject(hDC,fVertical);
  GetTextSize(hDC,plot.Ylabel,strlen(plot.Ylabel),&sText);
  TextOut(hDC,rWindow.left,(rWindow.bottom-rWindow.top+sText.cx)/2,plot.Ylabel,strlen(plot.Ylabel));
  SelectObject(hDC,fHorizontal);
  GetTextSize(hDC,plot.Xlabel,strlen(plot.Xlabel),&sText);
  TextOut(hDC,(rWindow.right-rWindow.left-sText.cx)/2,rWindow.bottom-sText.cy,plot.Xlabel,strlen(plot.Xlabel));
  if(plot.curves<1)
    Abort(__LINE__,"PlotCurves(): curves=%i <1",plot.curves);
  for(c=0;c<plot.curves;c++)
    if(plot.points[c]<2)
      Abort(__LINE__,"PlotCurves(): curve=%i, points=%i <2",c,plot.points[c]);
  Xm=Xx=plot.Xp[0];
  Ym=Yx=plot.Yp[0];
  for(c=i=0;c<plot.curves;c++)
    {
    for(p=0;p<plot.points[c];p++,i++)
      {
      Xm=min(Xm,plot.Xp[i]);
      Xx=max(Xx,plot.Xp[i]);
      Ym=min(Ym,plot.Yp[i]);
      Yx=max(Yx,plot.Yp[i]);
      }
    }
  if(Xm>=Xx)
    Abort(__LINE__,"PlotCurves(): X is degenerate");
  i=(int)(log10(Xx-Xm)-0.5);
  dX=pow(10,i);
  if(Xx-Xm>15*dX)
    dX*=5;
  else if(Xx-Xm>7*dX)
    dX*=2;
  X=Xm;
  Xm=dX;
  while(Xm<Xx)
    Xm+=dX;
  while(Xm>X)
    Xm-=dX;
  X=Xx;
  Xx=Xm;
  while(Xx<X)
    Xx+=dX;
  sXnum.cx=sXnum.cy=0;
  for(X=Xm;X<=Xx+dX/10;X+=dX)
    {
    sprintf(cbuf,"%lG",Xx);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    sXnum.cx=max(sXnum.cx,sText.cx);
    sXnum.cy=max(sXnum.cy,sText.cy);
    }
  if(Ym>=Yx)
    Abort(__LINE__,"PlotCurves(): Y is degenerate");
  i=(int)(log10(Yx-Ym)-0.5);
  dY=pow(10,i);
  if(Yx-Ym>15*dY)
    dY*=5;
  else if(Yx-Ym>7*dY)
    dY*=2;
  Y=Ym;
  Ym=dY;
  while(Ym<Yx)
    Ym+=dY;
  while(Ym>Y)
    Ym-=dY;
  Y=Yx;
  Yx=Ym;
  while(Yx<Y)
    Yx+=dY;
  sYnum.cx=sYnum.cy=0;
  for(Y=Ym;Y<=Yx+dY/10;Y+=dY)
    {
    sprintf(cbuf,"%lG",Yx);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    sYnum.cx=max(sYnum.cx,sText.cx);
    sYnum.cy=max(sYnum.cy,sText.cy);
    }
  GetTextSize(hDC,"0",1,&sTick);
  rPlot.left  =rWindow.left  +3*sTick.cx+sYnum.cx;
  rPlot.top   =rWindow.top   +sXnum.cy;
  rPlot.right =rWindow.right -sXnum.cx/2;
  rPlot.bottom=rWindow.bottom-sXnum.cy-sYnum.cy;
  Ax=(rPlot.right-rPlot.left)/(Xx-Xm);
  Bx=rPlot.left-Ax*Xm;
  Ay=(rPlot.top-rPlot.bottom)/(Yx-Ym);
  By=rPlot.bottom-Ay*Ym;
  for(X=Xm;X<Xx+dX/10;X+=dX)
    {
    x=nint(Ax*X+Bx);
    sprintf(cbuf,"%lG",X);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    TextOut(hDC,x-sText.cx/2,rPlot.bottom+sTick.cx/4,cbuf,strlen(cbuf));
    DrawLine(hDC,color[0],1,x,rPlot.bottom+sTick.cx/4,x,rPlot.top-1);
    }
  for(Y=Ym;Y<Yx+dY/10;Y+=dY)
    {
    y=nint(Ay*Y+By);
    sprintf(cbuf,"%lG",Y);
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    TextOut(hDC,rPlot.left-sText.cx-3*sTick.cx/4,y-sText.cy/2,cbuf,strlen(cbuf));
    DrawLine(hDC,color[0],1,rPlot.left-sTick.cx/2,y,rPlot.right+3,y);
    }
  rPlot.left++;
  rPlot.top++;
  hRgn=CreateRectRgnIndirect(&rPlot);
  SelectClipRgn(hDC,hRgn);
  for(c=i=0;c<plot.curves;c++)
    {
    x2=nint(Ax*plot.Xp[i]+Bx);
    y2=nint(Ay*plot.Yp[i]+By);
    for(p=1,++i;p<plot.points[c];p++,i++)
      {
      x1=x2;
      y1=y2;
      x2=nint(Ax*plot.Xp[i]+Bx);
      y2=nint(Ay*plot.Yp[i]+By);
      DrawLine(hDC,color[1+c%14],1,x1,y1,x2,y2);
      }
    }
  SetBkMode(hDC,OPAQUE);
  if(plot.legend&1)
    x=(rPlot.left+3*rPlot.right)/4;
  else
    x=rPlot.left+sTick.cx;
  if(plot.legend&2)
    y=rPlot.bottom-(2*plot.curves+1)*sTick.cy/2;
  else
    y=rPlot.top+sTick.cy/2;
  for(c=0;c<plot.curves;c++,y+=sXnum.cy)
    {
    SetTextColor(hDC,color[1+c%14]);
    TextOut(hDC,x,y,plot.label[c],strlen(plot.label[c]));
    }
  DeleteObject(hRgn);
  }

int WINAPI DrawProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_CLOSE)
    return(FALSE);
  if(wMsg==WM_COMMAND)
    return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  if(wMsg==WM_CREATE)
    return(FALSE);
  if(wMsg==WM_DESTROY)
    return(FALSE);
  if(wMsg==WM_PAINT)
    {
    int i;
    HDC hDC;
    PAINTSTRUCT pS;
    RECT cR;
    if((hDC=BeginPaint(hWnd,&pS))==NULL)
      Abort(__LINE__,"DrawProc(): can't get device context\nWin Err code %i",GetLastError());
    GetClientRect(hWnd,&cR);
    if(display_plot)
      PlotCurves(hDC,cR);
    else
      {
      for(i=0;i<2;i++)
        {
        FrameRect(hDC,&cR,GetStockObject(BLACK_BRUSH));
        cR.top++;
        cR.left++;
        cR.right--;
        cR.bottom--;
        }
      PaintBitmap(hDC,&cR,bPlant,0);
      }
    EndPaint(hWnd,&pS);
    return(FALSE);
    }
  if(wMsg==WM_TIMER)
    return(FALSE);
  return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  }

void DeallocateCurves()
  {
  int c;
  if(plot.points)
    free(plot.points);
  if(plot.label)
    {
    for(c=0;c<plot.curves;c++)
      free(plot.label[c]);
    free(plot.label);
    }
  if(plot.Xp)
    free(plot.Xp);
  if(plot.Yp)
    free(plot.Yp);
  }

void PositionWindows()
  {
  int x,y;
  RECT rc;
  GetClientRect(hMain,&rc);
  sizes.draw.x=rc.right-sizes.push.x-sizes.frame.x;
  sizes.draw.y=rc.bottom;
  x=y=0;
  MoveWindow(hPlot,x,y,sizes.draw.x,sizes.draw.y,TRUE);
  x+=sizes.draw.x+sizes.frame.x;
  MoveWindow(pReactor,x,y,sizes.push.x,sizes.push.y,TRUE);
  y+=sizes.push.y+sizes.frame.y;
  MoveWindow(pTurbine,x,y,sizes.push.x,sizes.push.y,TRUE);
  y+=sizes.push.y+sizes.frame.y;
  MoveWindow(pTower,x,y,sizes.push.x,sizes.push.y,TRUE);
  y+=sizes.push.y+sizes.frame.y;
  MoveWindow(pCondenser,x,y,sizes.push.x,sizes.push.y,TRUE);
  y=rc.bottom-sizes.button.y;
  MoveWindow(bExit,x,y,sizes.button.x,sizes.button.y,TRUE);
  y-=sizes.button.y+sizes.frame.y;
  MoveWindow(bAbout,x,y,sizes.button.x,sizes.button.y,TRUE);
  y-=sizes.button.y+sizes.frame.y;
  MoveWindow(bHelp,x,y,sizes.button.x,sizes.button.y,TRUE);
  }

void CenterWindowOnScreen(HWND hWnd)
  {
  int high,left,top,wide;
  RECT wr;
  GetWindowRect(hWnd,&wr);
  wide=wr.right-wr.left;
  high=wr.bottom-wr.top;
  left=(GetSystemMetrics(SM_CXSCREEN)-wide)/2;
  top=(GetSystemMetrics(SM_CYSCREEN)-high)/2;
  MoveWindow(hWnd,left,top,wide,high,TRUE);
  }

LRESULT WINAPI AboutProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_CLOSE)
    return(FALSE);
  if(wMsg==WM_COMMAND)
    {
    if(HIWORD(wParam)==BN_CLICKED||HIWORD(wParam)==BN_DOUBLECLICKED)
      return(EndDialog(hWnd,LOWORD(wParam)));
    else
      return(FALSE);
    }
  if(wMsg==WM_CTLCOLORBTN)
    {
    SendMessage((HWND)lParam,WM_SETFONT,(WPARAM)GetStockObject(SYSTEM_FONT),0);
    SetBkMode((HDC)wParam,TRANSPARENT);
    return((LRESULT)GetStockObject(LTGRAY_BRUSH));
    }
  if(wMsg==WM_CTLCOLORDLG)
    return((LRESULT)GetStockObject(LTGRAY_BRUSH));
  if(wMsg==WM_CTLCOLORSTATIC)
    {
    SelectObject((HDC)wParam,GetStockObject(ANSI_VAR_FONT));
    SetBkMode((HDC)wParam,TRANSPARENT);
    SetTextColor((HDC)wParam,0x000000);
    return((LRESULT)GetStockObject(LTGRAY_BRUSH));
    }
  if(wMsg==WM_DESTROY)
    return(TRUE);
  if(wMsg==WM_INITDIALOG)
    {
    CenterWindowOnScreen(hWnd);
    BringWindowToTop(hWnd);
    return(TRUE);
    }
  return(FALSE);
  }

/*******************************************************************************
  PLANT SIMULATION SPECIFIC OBJECTS
*******************************************************************************/

#define nz 3       /* number of condenser zones */
#define Trated 3621. /* rated reactor heat input [MWt] */
int gauge=20;      /* condenser tube gauge */
int Nplug=0;       /* number of plugged condenser tubes */
int Npump=4;       /* number of CCW pumps */
int Ntube=38025;   /* number of condenser tubes */
double bpmax=6.5;  /* maximum turbine backpressure [in.HgA] */
double cap=0.85;   /* cooling tower capability */
double Cc=0.95;    /* condenser cleanliness factor */
double Cm=0.94;    /* condenser tube material factor */
double Do=0.875;   /* condenser tube outside diameter [inch] */
double Qc=410000.; /* condenser cooling water flow rate [gpm] */
double Ltube=108.; /* condenser tube length (total) [ft] */
double bp[nz];     /* condenser pressure/turbine backpressure [in.HgA] */
double Di;         /* condenser tube inside diameter [inch] */
double heat=Trated;  /* reactor heat input [MWt] */
double rh=0.71;    /* ambient relative humidity */
double load=1200.; /* net generator output [MWe] */
double Tcold;      /* tower exit/condenser inlet temperature [F] */
double Thot[nz];   /* condenser cooling water temperatures by zone [F] */
double Tdb=65.;    /* ambient dry-bulb temperature [F] */
double Twb=55.;    /* ambient wet-bulb temperature [F] */

/* standard tubing wall thickness for gauges 12-24 */
double Wt[13]={0.109,0.095,0.083,0.072,0.065,0.058,0.049,
               0.042,0.035,0.032,0.028,0.025,0.022};

double hrcf(double pl,double bp) /* heatrate correction factor */
  {
  double c,Ts;
  Ts=fTsat(0.49115*bp);
  c=-16.915-0.970674*Ts+0.0216641*Ts*Ts-0.000147597*pow(Ts,3);
  c=c+3.48454E-7*pow(Ts,4)+246.238*pl-3.56102*pl*Ts+0.0216402*pl*Ts*Ts;
  c=c-3.70149E-5*pl*pow(Ts,3)-219.886*pl*pl+0.933872*pl*pl*Ts;
  c=c-0.0055049*pl*pl*Ts*Ts+179.522*pow(pl,3)+0.199748*pow(pl,3)*Ts;
  c=c-77.5384*pow(pl,4);
  return(c);
  }

double locf(double pl,double bp) /* load correction factor */
  {
  double cf;
  cf=hrcf(pl,bp);
  return(-100.*cf/(100.+cf));
  }

double GeneratorOutput(double heat,double*bp) /* generator output in MWe */
  {
  int z;
  double cf;
  for(cf=z=0;z<nz;z++)
    cf+=locf(heat/Trated,bp[z])/100.;
  return(((-1.05068E-5*heat+0.437274)*heat-182.008)*(1.+cf));
  }

double CondHrej(int zone,double heat,double bp) /* cond. heat reject. BTU/hr */
  {
  double rejt;
  rejt=(-3.33405054768E-7*heat+0.629284034809)*heat+89.6615694642;
  return((3412141.633*rejt/nz)*(1.+hrcf(heat/Trated,bp)/100.));
  }

double CondRise(int zone,double heat,double bp,double gpm) /* cond. rise F */
  {
  return(CondHrej(zone,heat,bp)/gpm/500.);
  }

double wtcf(double T) /* HEI water temperature correction factor */
  {
  return(0.376769/((((2.15973E-9*T-1.13681E-6)*T+0.000221828)*T-0.0195586)*T+1.));
  }

void FindBp(
  double Do,    /* tube outside diameter inch */
  int gauge,    /* tube gauge */
  double Cc,    /* condenser cleanliness in */
  double Cm,    /* tube material factor */
  double heat,  /* heat=reactor power in MWt */
  double gpm,   /* condenser flow in gpm */
  double Tcw,   /* condenser inlet water temp in F */
  double Thw[], /* condenser exit water temp in F */
  double bp[],  /* zone backpressures in Hg */
  double *load) /* generator output in MWe */
  {
  int iter,zone;
  double Af,bp_new,bp_old,Cv,Cw,Di,e,NTU,rise,Tsat,Tw[nz+1],Vw;
  Di=Do-2.*Wt[gauge-12];
  Cw=0.107*Do/Di/Di;
  Af=(Ntube-Nplug)*Di*Di*M_PI/4./144.;
  Vw=gpm/Af/7.48052/60.;
  Cv=263./sqrt(Vw);
  NTU=Cw*(Ltube/nz)*Cv*Cm*Cc/500.;
  for(zone=0;zone<nz;zone++)
    bp[zone]=2.2+zone;
  Tw[0]=Tcw;
  for(iter=0;iter<32;iter++)
    {
    for(zone=0;zone<nz;zone++)
      {
      rise=CondRise(zone,heat,bp[zone],gpm);
      e=exp(-NTU*wtcf(Tw[zone]));
      Tw[zone+1]=Tw[zone]+rise;
      Tsat=Tw[zone+1]+rise*e/(1.-e);
      bp_new=fPsat(Tsat)/0.49115;
      bp_old=bp[zone];
      bp[zone]=sqrt(bp_old*bp_new);
      }
    if(iter>2&&fabs(bp_old-bp_new)<0.001)break;
    }
  for(zone=0;zone<nz;zone++)
    Thw[zone]=Tw[zone+1];
  *load=GeneratorOutput(heat,bp);
  }

double Tower(double Twb,double gpm,double rh,double range)
  {
  double flow;
  flow=gpm/410000.;
  return(12.27487797+(0.7703383268+0.001639432789*Twb)*Twb
    +(-2.148767777-0.08288877122*Twb+1.771676191*rh)*rh
    +(0.3061704223-0.003077966102*Twb-0.002045529926*range)*range
    +(0.07235169492*range-0.1659431818*Twb-0.1646010524*rh
    +26.01944077-2.188920454*flow)*flow);
  }

double FindLoad(
  double heat,  /* heat=reactor power [MWt] */
  double gpm,   /* condenser flow [gpm] */
  double Twb,   /* wet-bulb temperature [F] */
  double rh,    /* relative humidity */
  double cap,   /* tower capability */
  double*Tcw,   /* condenser inlet water temperature [F] */
  double Thw[], /* condenser exit water temperature [F] */
  double bp[])  /* zone backpressures [in.HgA] */
  {
  int iter;
  double load,range,Tc;
  range=35.2*heat/3619.;
  Tc=max(32.,Twb+range/2.);
  for(iter=0;iter<32;iter++)
    {
    FindBp(Do,gauge,Cc,Cm,heat,gpm,Tc,Thw,bp,&load);
    range=(gpm*Thw[1]+25000.*25.)/(gpm+25000.)-Tc;
    *Tcw=Tower(Twb,(gpm+25000.)/cap,rh,range);
    if(iter>2&&fabs(*Tcw-Tc)<0.01)break;
    Tc=*Tcw;
    }
  return(load);
  }

double Cf(double Re,double e) /* Colebrook's formula for friction factor */
  {
  double f;
  int iter;
  if(Re<2300.)return(64./Re);
  f=0.01;
  for(iter=0;iter<5;iter++)
    {
    f=1.14-2.*(log10(e)+log10(1.+9.3/Re/e/sqrt(f)));
    f=1./f/f;
    }
  if(Re<5500.)
    f=64./Re+(log10(Re)-log10(2300.))/(log10(5000.)-log10(2300.))*(f-64./Re);
  return(f);
  }

double PumpCurve(double gpm) /* pump head in feet from flow in gpm */
  {
  return((142.5-gpm/1271.)/(1.-gpm/318300.));
  }

double Resistance(double gpm) /* CCW flow resistance in ft */
  {
  return((gpm/46041.+1.)*gpm/87126.+44.622);
  }

#define gravity 32.174

double FindFlow() /* CCW flow in gpm */
  {
  int iter;
  double e=0.00006,mu=0.0000122;
  double f,head,Q,Q1,Q2,Re,V;
  Di=Do-2.*Wt[gauge-12];
  Q1=0;
  Q2=160000.*Npump;
  for(iter=0;iter<25;iter++)
    {
    Q=(Q1+Q2)/2.;
    V=Q/Di/Di/(Ntube-Nplug)/M_PI*4*144/448.83;
    Re=V*Di/12/mu;
    f=Cf(Re,e);
    head=41.646+23.6*pow(Q/410000.,2)+f*Ltube/Di*V*V*12./2./gravity;
    if(head<PumpCurve(Q/Npump))
      Q1=Q;
    else
      Q2=Q;
    }
  return(Q);
  }

void printc(char*format,...)
  {
  char bufr[128],*ptr;
  DWORD l;
  va_list arg_marker;
  if(!stdOutput)
    return;
  va_start(arg_marker,format);
  if(vsprintf(bufr,format,arg_marker)>=sizeof(bufr))
    FatalAppExit(0,"printc(): buffer overflow");
  if((ptr=strchr(bufr,'\n'))!=NULL)
    strcpy(ptr,"\r\n");
  l=(DWORD)strlen(bufr);
  WriteConsole(stdOutput,bufr,l,&l,NULL);
  }

void Simulation()
  {
  int bisect;
  double gpm,hmin,hmax;
  gpm=FindFlow(Npump);
  hmin=0.;
  hmax=2.*heat;
  for(bisect=0;bisect<32;bisect++)
    {
    heat=(hmin+hmax)/2.;
    load=FindLoad(heat,gpm,Twb,rh,cap,&Tcold,Thot,bp);
    if(bp[nz-1]<bpmax)
      {
      if(bisect==0)
        break;
      hmin=heat;
      }
    else
      hmax=heat;
    }
  printc("%3.0lf %4.2lf %4.0lf %4.0lf %5.1lf %5.1lf %4.1lf %4.2lf %4.2lf %4.2lf\n",Tdb,rh,heat,load,Tcold,Thot[2],Thot[2]-Tcold,bp[0],bp[1],bp[2]);
  }

typedef struct{
  double min;
  double max;
  int pts;
  }VAR;

void CreateReactorPlot()
  {
  int i,j,z;
  double bpj,ht[4];
  VAR BP={1.5,5.0,21};
  plot.Title ="Impact of Backpressure on Capacity";
  plot.Xlabel="Backpressure [in.HgA]";
  plot.Ylabel="Generator Output [MWe]";
  DeallocateCurves();
  plot.curves=4;
  plot.points=calloc(plot.curves,sizeof(int));
  plot.points[0]=
  plot.points[1]=
  plot.points[2]=
  plot.points[3]=BP.pts;
  plot.legend=2;
  plot.label=calloc(plot.curves,sizeof(char*));
  for(j=0;j<plot.curves;j++)
    {
    ht[j]=(100-10*j)*Trated/100.;
    sprintf(cbuf,"%4.0lf MWt",ht[j]);
    plot.label[j]=malloc(strlen(cbuf)+1);
    strcpy(plot.label[j],cbuf);
    }
  plot.Xp=calloc(plot.curves*BP.pts,sizeof(double));
  plot.Yp=calloc(plot.curves*BP.pts,sizeof(double));
  for(i=0;i<BP.pts;i++)
    {
    bpj=BP.min+i*(BP.max-BP.min)/(BP.pts-1);
    for(j=0;j<plot.curves;j++)
      {
      plot.Xp[BP.pts*j+i]=bpj;
      for(z=0;z<nz;z++)
        bp[z]=bpj;
      plot.Yp[BP.pts*j+i]=GeneratorOutput(ht[j],bp);
      }
    }
  }

void CreateTurbinePlot()
  {
  int i,j;
  double bp;
  VAR BP={0.5,5.5,31};
  plot.Title ="Impact of Backpressure on Heat Rate";
  plot.Xlabel="Zone 2 Backpressure [in.HgA]";
  plot.Ylabel="Heat Rate Correction [%]";
  DeallocateCurves();
  plot.curves=4;
  plot.points=calloc(plot.curves,sizeof(int));
  plot.points[0]=
  plot.points[1]=
  plot.points[2]=
  plot.points[3]=BP.pts;
  plot.legend=3;
  plot.label=calloc(plot.curves,sizeof(char*));
  for(j=0;j<4;j++)
    {
    sprintf(cbuf,"%3i%% steamflow",(4-j)*25);
    plot.label[j]=malloc(strlen(cbuf)+1);
    strcpy(plot.label[j],cbuf);
    }
  plot.Xp=calloc(plot.curves*BP.pts,sizeof(double));
  plot.Yp=calloc(plot.curves*BP.pts,sizeof(double));
  for(i=0;i<BP.pts;i++)
    {
    bp=BP.min+i*(BP.max-BP.min)/(BP.pts-1);
    for(j=0;j<plot.curves;j++)
      {
      plot.Xp[BP.pts*j+i]=bp;
      plot.Yp[BP.pts*j+i]=hrcf((4-j)/4.,bp);
      }
    }
  }

void CreateTowerPlot()
  {
  int i,j;
  double gpm=410000.,rh,range=38.,Twb;
  VAR TWB={30.,80.,11};
  plot.Title ="Cooling Tower Performance";
  plot.Xlabel="Wet-Bulb Temperature [F]";
  plot.Ylabel="Cooling Tower Exit Temperature [F]";
  DeallocateCurves();
  plot.curves=4;
  plot.points=calloc(plot.curves,sizeof(int));
  plot.points[0]=
  plot.points[1]=
  plot.points[2]=
  plot.points[3]=TWB.pts;
  plot.legend=0;
  plot.label=calloc(plot.curves,sizeof(char*));
  for(j=0;j<4;j++)
    {
    sprintf(cbuf,"RH=%2i%%",30+20*j);
    plot.label[j]=malloc(strlen(cbuf)+1);
    strcpy(plot.label[j],cbuf);
    }
  plot.Xp=calloc(plot.curves*TWB.pts,sizeof(double));
  plot.Yp=calloc(plot.curves*TWB.pts,sizeof(double));
  for(i=0;i<TWB.pts;i++)
    {
    Twb=TWB.min+i*(TWB.max-TWB.min)/(TWB.pts-1);
    for(j=0;j<plot.curves;j++)
      {
      rh=0.30+j*0.2;
      plot.Xp[TWB.pts*j+i]=Twb;
      plot.Yp[TWB.pts*j+i]=Tower(Twb,410000.,rh,range);
      }
    }
  }

void CreateCondenserPlot()
  {
  int i,j;
  double bp[nz],ht[4],load,Tin,Thw[nz];
  VAR TIN={35.,115.,17};
  plot.Title ="Condenser Performance";
  plot.Xlabel="Cooling Water Inlet Temperature [F]";
  plot.Ylabel="Backpressure [in.HgA]";
  DeallocateCurves();
  plot.curves=4;
  plot.points=calloc(plot.curves,sizeof(int));
  plot.points[0]=
  plot.points[1]=
  plot.points[2]=
  plot.points[3]=TIN.pts;
  plot.legend=0;
  plot.label=calloc(plot.curves,sizeof(char*));
  for(j=0;j<plot.curves;j++)
    {
    ht[j]=(100-10*j)*Trated/100.;
    sprintf(cbuf,"%4.0lf MWt",ht[j]);
    plot.label[j]=malloc(strlen(cbuf)+1);
    strcpy(plot.label[j],cbuf);
    }
  plot.Xp=calloc(plot.curves*TIN.pts,sizeof(double));
  plot.Yp=calloc(plot.curves*TIN.pts,sizeof(double));
  for(i=0;i<TIN.pts;i++)
    {
    Tin=TIN.min+i*(TIN.max-TIN.min)/(TIN.pts-1);
    for(j=0;j<plot.curves;j++)
      {
      FindBp(Do,gauge,Cc,Cm,ht[j],Qc,Tin,Thw,bp,&load);
      plot.Xp[TIN.pts*j+i]=Tin;
      plot.Yp[TIN.pts*j+i]=bp[nz-1];
      }
    }
  }

int WINAPI PushProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_LBUTTONDBLCLK)
    {
    PostMessage(hWnd,WM_LBUTTONDOWN,wParam,lParam);
    PostMessage(hWnd,WM_LBUTTONUP,wParam,lParam);
    }
  if(wMsg==WM_LBUTTONDOWN)
    {
    SetWindowLong(hWnd,GWL_ID,TRUE);
    InvalidateRect(hWnd,NULL,TRUE);
    UpdateWindow(hWnd);
    return(FALSE);
    }
  if(wMsg==WM_LBUTTONUP)
    {
    SetWindowLong(hWnd,GWL_ID,FALSE);
    InvalidateRect(hWnd,NULL,TRUE);
    UpdateWindow(hWnd);
    display_plot^=1;
    if(display_plot)
      {
      ShowWindow(pCondenser,SW_HIDE);
      ShowWindow(pTurbine  ,SW_HIDE);
      ShowWindow(pTower    ,SW_HIDE);
      ShowWindow(pReactor  ,SW_HIDE);
      ShowWindow(hWnd      ,SW_SHOW);
      if(hWnd==pCondenser)
        CreateCondenserPlot();
      else if(hWnd==pTurbine)
        CreateTurbinePlot();
      else if(hWnd==pTower)
        CreateTowerPlot();
      else if(hWnd==pReactor)
        CreateReactorPlot();
      }
    else
      {
      ShowWindow(pCondenser,SW_SHOW);
      ShowWindow(pTower    ,SW_SHOW);
      ShowWindow(pTurbine  ,SW_SHOW);
      ShowWindow(pReactor  ,SW_SHOW);
      }
    InvalidateRect(hPlot,NULL,TRUE);
    return(FALSE);
    }
  if(wMsg==WM_PAINT)
    {
    int i,x,y;
    HDC hDC;
    HPEN hPenO,hPenS,hPenW;
    PAINTSTRUCT pS;
    POINT P;
    RECT cR;
    static SIZE sText;
    if((hDC=BeginPaint(hWnd,&pS))==NULL)
      Abort(__LINE__,"PushProc(): can't get device context\nWin Err code %i",GetLastError());
    GetClientRect(hWnd,&cR);
    for(i=0;i<2;i++)
      {
      FrameRect(hDC,&cR,GetStockObject(BLACK_BRUSH));
      cR.top++;
      cR.left++;
      cR.right--;
      cR.bottom--;
      }
    if(GetWindowLong(hWnd,GWL_ID))
      {
      hPenW=CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNSHADOW));
      hPenS=CreatePen(PS_SOLID,1,GetSysColor(COLOR_WINDOW));
      }
    else
      {
      hPenW=CreatePen(PS_SOLID,1,GetSysColor(COLOR_WINDOW));
      hPenS=CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNSHADOW));
      }
    for(i=0;i<3;i++)
      {
      hPenO=SelectObject(hDC,hPenW);
      MoveToEx(hDC,cR.left,cR.top,&P);
      LineTo(hDC,cR.right,cR.top);
      MoveToEx(hDC,cR.left,cR.top,&P);
      LineTo(hDC,cR.left,cR.bottom);
      hPenW=SelectObject(hDC,hPenS);
      MoveToEx(hDC,cR.left,cR.bottom-1,&P);
      LineTo(hDC,cR.right,cR.bottom-1);
      MoveToEx(hDC,cR.right-1,cR.top,&P);
      LineTo(hDC,cR.right-1,cR.bottom);
      hPenS=SelectObject(hDC,hPenO);
      cR.top++;
      cR.left++;
      cR.right--;
      cR.bottom--;
      }
    DeleteObject(hPenS);
    DeleteObject(hPenW);
    PaintBitmap(hDC,&cR,(BITMAPINFOHEADER*)GetWindowLong(hWnd,GWL_USERDATA),1);
    SetBkMode(hDC,TRANSPARENT);
    GetWindowText(hWnd,cbuf,sizeof(cbuf));
    GetTextSize(hDC,cbuf,strlen(cbuf),&sText);
    x=(cR.right+cR.left-sText.cx)/2;
    y=cR.bottom-sText.cy-2;
    SetTextColor(hDC,RGB(0xFF,0xFF,0xFF));
    TextOut(hDC,x+1,y-1,cbuf,strlen(cbuf));
    TextOut(hDC,x+1,y+1,cbuf,strlen(cbuf));
    TextOut(hDC,x-1,y+1,cbuf,strlen(cbuf));
    TextOut(hDC,x-1,y-1,cbuf,strlen(cbuf));
    SetTextColor(hDC,RGB(0x00,0x00,0x00));
    TextOut(hDC,x,y,cbuf,strlen(cbuf));
    EndPaint(hWnd,&pS);
    return(FALSE);
    }
  return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  }

int WINAPI MainProc(HWND hWnd,DWORD wMsg,WPARAM wParam,LPARAM lParam)
  {
  if(wMsg==WM_CLOSE)
    {
    PostQuitMessage(0);
    return(FALSE);
    }
  if(wMsg==WM_COMMAND)
    {
    if(HIWORD(wParam)==BN_CLICKED)
      {
      if(LOWORD(wParam)==PUSH_ABOUT)
        DialogBox(hInst,"ABOUT",hMain,(DLGPROC)AboutProc);
      else if(LOWORD(wParam)==PUSH_EXIT)
        PostQuitMessage(0);
      else if(LOWORD(wParam)==PUSH_HELP)
        WinHelp(hMain,"WattsBar.hlp",HELP_CONTENTS,0);
      else if(LOWORD(wParam)==PUSH_MAIN)
        PostMessage(pReactor,WM_LBUTTONDBLCLK,0,0);
      return(FALSE);
      }
    return(DefWindowProc(hWnd,wMsg,wParam,lParam));
    }
  if(wMsg==WM_CREATE)
    return(FALSE);
  if(wMsg==WM_CTLCOLORDLG)
    return((DWORD)GetStockObject(LTGRAY_BRUSH));
  if(wMsg==WM_DESTROY)
    return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  if(wMsg==WM_PAINT)
    {
    HDC hDC;
    PAINTSTRUCT pS;
    if((hDC=BeginPaint(hWnd,&pS))==NULL)
      Abort(__LINE__,"can't get device context\nWin Err code %i",GetLastError());
    EndPaint(hWnd,&pS);
    return(FALSE);
    }
  if(wMsg==WM_SIZE)
    {
    PositionWindows();
    return(FALSE);
    }
  if(wMsg==WM_TIMER)
    return(FALSE);
  return(DefWindowProc(hWnd,wMsg,wParam,lParam));
  }

void CreateFonts()
  {
  fBold      =CreateStandardFont("Arial",FW_HEAVY , 0);
  fHorizontal=CreateStandardFont("Arial",FW_MEDIUM, 0);
  fVertical  =CreateStandardFont("Arial",FW_MEDIUM,90);
  }

void*GetResource(char*rName,char*type)
  {
  void*rLock;
  HGLOBAL rLoad;
  HRSRC rFind;
  if((rFind=FindResource(hInst,rName,type))==NULL)
    Abort(__LINE__,"can't find resource %s\nWin Err code %i",rName,GetLastError());
  if((rLoad=LoadResource(hInst,rFind))==NULL)
    Abort(__LINE__,"can't load resource %s\nWin Err code %i",rName,GetLastError());
  if((rLock=LockResource(rLoad))==NULL)
    Abort(__LINE__,"can't lock resource %s\nWin Err code %i",rName,GetLastError());
  return(rLock);
  }

BITMAPINFOHEADER*GetBitmap(char*name)
  {
  return((BITMAPINFOHEADER*)((BYTE*)GetResource(name,RT_BITMAP)));
  }

void LoadBitmaps()
  {
  bCondenser=GetBitmap("CONDENSER");
  bPlant    =GetBitmap("PLANT");
  bTurbine  =GetBitmap("TURBINE");
  bTower    =GetBitmap("TOWER");
  bReactor  =GetBitmap("REACTOR");
  }

void CreateWindows()
  {
  sizes.screen.x=GetSystemMetrics(SM_CXSCREEN);
  sizes.screen.y=GetSystemMetrics(SM_CYSCREEN);
  sizes.plant.x=bPlant->biWidth;
  sizes.plant.y=bPlant->biHeight;
  sizes.frame.x=GetSystemMetrics(SM_CXFRAME)/2;
  sizes.frame.y=GetSystemMetrics(SM_CYFRAME)/2;
  sizes.button.x=GetSystemMetrics(SM_CXMINTRACK)+2*sizes.frame.x;
  sizes.button.y=GetSystemMetrics(SM_CYVTHUMB)+2*sizes.frame.y;
  sizes.push.x=sizes.button.x;
  sizes.push.y=3*(sizes.push.x-10)/4+10;
  sizes.client.x=sizes.plant.x+sizes.push.x+sizes.frame.x+8;
  sizes.client.y=sizes.plant.y+8;
  sizes.window.x=sizes.client.x+2*sizes.frame.x;
  sizes.window.y=sizes.client.y+2*sizes.frame.x+GetSystemMetrics(SM_CYCAPTION);
  if((hMain=CreateWindow("MAIN","Watts Bar Nuclear Heat Rejection System Model",
      WS_OVERLAPPEDWINDOW|WS_VISIBLE,
      (sizes.screen.x-sizes.window.x)/2,
      (sizes.screen.y-sizes.window.y)/2,
      sizes.window.x,sizes.window.y,
      NULL,NULL,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create main window\nWin Err code %i",GetLastError());
  if((hPlot=CreateWindow("DRAW",NULL,
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create draw window\nWin Err code %i",GetLastError());
  if((pReactor=CreateWindow("PUSH","Reactor",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create reactor button\nWin Err code %i",GetLastError());
  SetWindowLong(pReactor,GWL_USERDATA,(long)bReactor);
  if((pTurbine=CreateWindow("PUSH","Steam System",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create turbine button\nWin Err code %i",GetLastError());
  SetWindowLong(pTurbine,GWL_USERDATA,(long)bTurbine);
  if((pTower=CreateWindow("PUSH","Cooling Towers",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create cooling tower button\nWin Err code %i",GetLastError());
  SetWindowLong(pTower,GWL_USERDATA,(long)bTower);
  if((pCondenser=CreateWindow("PUSH","Condenser",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|SS_BLACKFRAME,
      0,0,0,0,hMain,0,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create Condenser button\nWin Err code %i",GetLastError());
  SetWindowLong(pCondenser,GWL_USERDATA,(long)bCondenser);
  if((bExit=CreateWindow("BUTTON","Exit",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|BS_PUSHBUTTON,
      0,0,0,0,hMain,(void*)PUSH_EXIT,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create exit button\nWin Err code %i",GetLastError());
  if((bAbout=CreateWindow("BUTTON","About",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|BS_PUSHBUTTON,
      0,0,0,0,hMain,(void*)PUSH_ABOUT,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create about button\nWin Err code %i",GetLastError());
  if((bHelp=CreateWindow("BUTTON","Help",
      WS_CHILD|WS_CLIPSIBLINGS|WS_VISIBLE|BS_PUSHBUTTON,
      0,0,0,0,hMain,(void*)PUSH_HELP,hInst,NULL))==NULL)
    Abort(__LINE__,"can't create help button\nWin Err code %i",GetLastError());
  PositionWindows();
  }

void Register(char*cName,void*wProc,HBRUSH hBrush,char*cursor)
  {
  WNDCLASS wc;
  memset(&wc,0,sizeof(WNDCLASS));
  wc.hInstance    =hInst;
  wc.hIcon        =LoadIcon(hInst,"ICON");
  wc.style        =CS_HREDRAW|CS_VREDRAW;
  wc.lpszClassName=cName;
  wc.lpfnWndProc  =(WNDPROC)wProc;
  wc.hbrBackground=hBrush;
  wc.hCursor      =LoadCursor(NULL,cursor);
  if(!RegisterClass(&wc))
    Abort(__LINE__,"can't register [%s] class\nWin Err code %i",cName,GetLastError());
  }

void RegisterClasses()
  {
  Register("MAIN",MainProc,GetStockObject(LTGRAY_BRUSH),IDC_ARROW);
  Register("DRAW",DrawProc,GetStockObject(LTGRAY_BRUSH),IDC_NO   );
  Register("PUSH",PushProc,GetStockObject(DKGRAY_BRUSH),IDC_ARROW);
  }

void Tables()
  {
  double ht;
  printc("pumps  gpm\n");
  for(Npump=1;Npump<=4;Npump++)
    printc("  %i %6.0lf\n",Npump,FindFlow());
  printc("Tdb  RH   MWt  MWe  Tcw   Thw  rise bp1  bp2  bp3\n");
  Tdb=59.;
  rh=0.6;
  Twb=fTwdrh(14.696,Tdb,rh);
  for(ht=100.*(int)(Trated/200.);ht<=Trated;ht+=100.)
    {
    heat=ht;
    Simulation();
    }
  printc("Tdb  RH   MWt  MWe  Tcw   Thw  rise bp1  bp2  bp3\n");
  for(Tdb=30.;Tdb<110.1;Tdb+=10.)
    {
    for(rh=0.2;rh<1.01;rh+=0.2)
      {
      Twb=fTwdrh(14.696,Tdb,rh);
      heat=Trated;
      Simulation();
      }
    }
  }

void BatchMode()
  {
  FILE*fi,*fo;
  if(!AllocConsole())
    Abort(__LINE__,"can't create console\nWin Err code %i",GetLastError());
  CenterWindowOnScreen(GetConsoleWindow());
  BringWindowToTop(GetConsoleWindow());
  stdOutput=GetStdHandle(STD_OUTPUT_HANDLE);
  stdInput=GetStdHandle(STD_INPUT_HANDLE);
  if((fo=fopen("WattsBar.csv","wt"))==NULL)
    Abort(__LINE__,"can't create output file");
  Tables();
  if((fi=fopen(__argv[1],"rt"))==NULL)
    Abort(__LINE__,"can't open input file [%s]",__argv[1]);
  while(fgets(cbuf,sizeof(cbuf),fi))
    {
    if(sscanf(cbuf,"%lf%*[ ,\t]%lf",&Tdb,&Twb)==2)
      {
      rh=fRHdwb(14.696,Tdb,Twb);
      heat=Trated;
      cap=0.85;
      bpmax=4.5;
      Simulation();
      fprintf(fo,"%.1lf,%.1lf,%.1lf,%.1lf,%.3lf,%.3lf,%.3lf\n",Tdb,Twb,heat,load,bp[0],bp[1],bp[2]);
      }
    }
  fclose(fi);
  fclose(fo);
  BringWindowToTop(GetConsoleWindow());
  MessageBox(NULL,"click OK to exit program","batch run complete",MB_SYSTEMMODAL|MB_ICONSTOP|MB_OK);
  FreeConsole();
  }

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevious,char*lpszLine,int nShow)
  {
  MSG msg;
  hInst=hInstance;
  if(__argc>1)
    {
    BatchMode();
    ExitProcess(0);
    }
  LoadBitmaps();
  CreateFonts();
  RegisterClasses();
  CreateWindows();
  while(GetMessage(&msg,NULL,0,0))
    {
    if(msg.message==WM_SYSKEYDOWN&&msg.wParam==VK_F4)
      break;
    if(!TranslateMessage(&msg))
      DispatchMessage(&msg);
    }
  return(0);
  }
